# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""Script to reinitialize surveil."""

import optparse
import sys

import influxdb
import pecan
from pecan import configuration
import pymongo
from six.moves import configparser
import surveilclient.client as sc

from surveil.cmd import pack_upload


def main():
    parser = optparse.OptionParser()
    parser.add_option('-d', '--demo',
                      default=False,
                      dest='demo',
                      help="Create fake hosts in Alignak",
                      action="store_true")
    parser.add_option('-i', '--influxdb',
                      default=False,
                      dest='influxdb',
                      help="Pre-create the InfluxDB database",
                      action='store_true')
    parser.add_option('-m', '--mongodb',
                      default=False,
                      dest='mongodb',
                      help="Drop the existing Alignak MongoDB database",
                      action='store_true')
    parser.add_option('-p', '--packs',
                      default=False,
                      dest='packs',
                      help="Upload/Update configuration packs to MongoDB",
                      action='store_true')
    parser.add_option('--pecan_config', '-P',
                      default='/etc/surveil/config.py',
                      dest='pecan_config',
                      help='Pecan config file (config.py)')
    parser.add_option('--config_file', '-c',
                      default='/etc/surveil/surveil.cfg',
                      dest='config_file',
                      help='Pecan config file (surveil.cfg)')

    opts, _ = parser.parse_args(sys.argv)

    surveil_api_url = 'http://localhost:5311/v2'
    surveil_auth_url = 'http://localhost:5311/v2/auth'
    surveil_api_version = '2_0'

    config = configparser.ConfigParser()
    config.read(opts.config_file)
    surveil_cfg = {"surveil_api_config":
                   dict([i for i in config.items("surveil")])}
    configuration.set_config(surveil_cfg, overwrite=True)
    configuration.set_config(opts.pecan_config, overwrite=False)

    cli_surveil = sc.Client(surveil_api_url,
                            auth_url=surveil_auth_url,
                            version=surveil_api_version)

    # Create a basic config in mongodb
    mongo = pymongo.MongoClient(pecan.conf.surveil_api_config['mongodb_uri'])

    if opts.mongodb is True:
        # Drop the current shinken config
        print("Dropping existing Alignak MongoDB database...")
        mongo.drop_database('shinken')

    if opts.influxdb is True:
        print("Pre-creating InfluxDB database...")
        # Create the InfluxDB database
        influx_client = influxdb.InfluxDBClient.from_DSN(
            pecan.conf.surveil_api_config['influxdb_uri']
        )

        databases = influx_client.get_list_database()
        if not any(db['name'] == influx_client._database for db in databases):
            influx_client.create_database(influx_client._database)

    if opts.packs:
        print ("Uploading packs...")
        # Load the shinken packs
        pack_upload.upload_pack(
            "/usr/share/monitoring/packs/sfl/openstack-keystone-http/",
            cli_surveil
        )

        pack_upload.upload_pack(
            "/usr/share/monitoring/packs/sfl/openstack-glance-http/",
            cli_surveil
        )
        pack_upload.upload_pack(
            "/usr/share/monitoring/packs/sfl/generic-host/",
            cli_surveil
        )
        pack_upload.upload_pack(
            "/usr/share/monitoring/packs/sfl/linux-system-nrpe/",
            cli_surveil
        )
        pack_upload.upload_pack(
            "/usr/share/monitoring/packs/sfl/openstack-nova-http/",
            cli_surveil
        )
        pack_upload.upload_pack(
            "/usr/share/monitoring/packs/sfl/openstack-cinder-http/",
            cli_surveil
        )
        pack_upload.upload_pack(
            "/usr/share/monitoring/packs/sfl/openstack-host/",
            cli_surveil
        )

    # if --demo is specified, you get more hosts.
    if opts.demo is True:
        print("Creating demo configuration...")
        # shinken's ws-arbiter
        cli_surveil.config.hosts.create(
            use=["generic-host"],
            contact_groups=["admins"],
            host_name="ws-arbiter",
            address="localhost"
        )
        cli_surveil.config.services.create(
            check_command="check_tcp!7760",
            check_interval="5",
            check_period="24x7",
            contact_groups=["admins"],
            contacts=["admin"],
            host_name=["ws-arbiter"],
            max_check_attempts="5",
            notification_interval="30",
            notification_period="24x7",
            retry_interval="3",
            service_description="check-ws-arbiter"
        )

        # Passive checks host
        cli_surveil.config.hosts.create(
            host_name='passive_check_host',
            use=['generic-host'],
            address='127.0.0.1',
            passive_checks_enabled='1'
        )

        # Linux-keystone template
        cli_surveil.config.hosts.create(
            host_name='test_keystone',
            use=['openstack-keystone-http'],
            address='127.0.0.1',
            custom_fields={
                "_OS_AUTH_URL": "bla",
                "_OS_USERNAME": "bli",
                "_OS_PASSWORD": "blo",
                "_OS_TENANT":   "blu",
                "_KS_SERVICES": "bly",
                "parents": "localhost"
            }
        )

        # openstack-host template
        cli_surveil.config.hosts.create(
            host_name='openstackceilometer-host',
            use=['openstack-host'],
            address='127.0.0.1',
            custom_fields={
                "_OS_AUTH_URL": "bla",
                "_OS_USERNAME": "bli",
                "_OS_PASSWORD": "blo",
                "_OS_TENANT_NAME":   "blu",
                "_KS_SERVICES": "bly",
                "parents": "localhost"
            }
        )

        # DOWN HOST (cant resolve)
        cli_surveil.config.hosts.create(
            host_name='srv-apache-01',
            use=['linux-system-nrpe'],
            address='srv-apache-01',
            custom_fields={
                "_TRAFFICLIMIT": "100000",
            }
        )

        # DOWN, and parent down (Network outage)
        cli_surveil.config.hosts.create(
            host_name='myparentisdown',
            address='dfgsdgsdgf',
            parents=['srv-apache-01'],
        )

        # UP host, no template
        cli_surveil.config.hosts.create(
            host_name='google.com',
            address='google.com'
        )

        # NRPE host, UP
        cli_surveil.config.hosts.create(
            host_name='srv-monitoring-01',
            use=['linux-system-nrpe'],
            address='127.0.0.1',
            custom_fields={
                "_TRAFFICLIMIT": "500000",
            }
        )

        # Has parent, UP
        cli_surveil.config.hosts.create(
            host_name='sw-iwebcore-01',
            parents=['srv-monitoring-01'],
            use=['generic-host'],
            address='127.0.0.1',
            custom_fields={
                "_TRAFFICLIMIT": "200000",
            }
        )

        # Has chain of 2 parents, UP
        cli_surveil.config.hosts.create(
            host_name='srv-ldap-01',
            parents=['sw-iwebcore-01'],
            use=['generic-host'],
            address='127.0.0.1',
            custom_fields={
                "_TRAFFICLIMIT": "5000000",
            }
        )

        # UP host with down service
        cli_surveil.config.hosts.create(
            use=["generic-host"],
            contact_groups=["admins"],
            host_name="myserviceisdown",
            address="localhost"
        )
        cli_surveil.config.services.create(
            check_command="check_tcp!4553",
            check_interval="5",
            check_period="24x7",
            contact_groups=["admins"],
            contacts=["admin"],
            host_name=["myserviceisdown"],
            max_check_attempts="5",
            notification_interval="30",
            notification_period="24x7",
            retry_interval="3",
            service_description="iamadownservice"
        )

        cli_surveil.config.businessimpactmodulations.create(
            business_impact_modulation_name="demoBIM",
            business_impact=42,
            modulation_period="24x7"
        )

        cli_surveil.config.checkmodulations.create(
            checkmodulation_name="demoCheckModulation",
            check_command="42",
            check_period="24x7"
        )

        cli_surveil.config.commands.create(
            command_name="demoCommand",
            command_line="ls --all",
            module_type="active"
        )

        cli_surveil.config.contacts.create(
            contact_name="demoContact",
            host_notifications_enabled=1,
            service_notifications_enabled=0,
            host_notification_period="24x6",
            service_notification_period="12x7",
            host_notification_options=["mail", "sms"],
            service_notification_options=["mail"],
            host_notification_commands=["sms admin"],
            service_notification_commands=["send mail to admin"],
            email="surveil.demo@gmail.com",
            pager="12",
            can_submit_commands=1,
            is_admin=0,
            retain_status_information=1,
            retain_nonstatus_information=365,
            min_business_impact=24
        )

        cli_surveil.config.contactgroups.create(
            contactgroup_name="BDFL",
            members=["demoContact"],
            alias="group Alias",
            contactgroup_members=["Linus", "Richard"]
        )

        cli_surveil.config.hostgroups.create(
            hostgroup_name="OpenStack01",
            members=['sw-iwebcore-01', 'srv-monitoring-01'],
            alias="group Alias",
            hostgroup_members=["AWS01", "Azure01"],
            notes="notes",
            notes_url="http://....notes.com",
            action_url="mail"
        )

        cli_surveil.config.macromodulations.create(
            macromodulation_name="Demo macroModulation",
            modulation_period="24x7",
            macro={"Linus": "Linux", "Richard": "Gnu"}
        )

        cli_surveil.config.notificationways.create(
            notificationway_name="Demo motificationway",
            host_notification_period="24x7",
            service_notification_period="12x7",
            host_notification_options=["mail", "sms"],
            service_notification_options=["mail"],
            host_notification_commands=["sms admin"],
            service_notification_commands=["send mail to admin"],
            min_business_impact="24"
        )

        cli_surveil.config.realms.create(
            realm_name="Mordor",
            realm_members=["Frodon", "Gandalf", "Sauron"],
            default=42
        )

        cli_surveil.config.servicegroups.create(
            servicegroup_name="check Openstack",
            members=["check-ws-arbiter"],
            alias="group Alias",
            servicegroup_members=["check Keystone", "check Nova"],
            notes="notes",
            notes_url="http://....notes.com",
            action_url="mail"
        )

        cli_surveil.config.timeperiods.create(
            timeperiod_name="Demo TimePeriod",
            exclude=["24x7", "11x12"],
            periods={"day": "demo", "night": "demoNight"},
            alias="aliasGroup"
        )
    # Reload the shinken config
    cli_surveil.config.reload_config()

if __name__ == "__main__":
    main()



# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from pecan import rest

from surveil.api.controllers.v2.status.events import events as v2_events
from surveil.api.controllers.v2.status.hosts import hosts as v2_hosts
from surveil.api.controllers.v2.status.services import services as v2_services


class StatusController(rest.RestController):
    hosts = v2_hosts.HostsController()
    services = v2_services.ServicesController()
    events = v2_events.EventsController()



# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from __future__ import print_function

import os
import unittest

from surveil.tests import base
from surveil.tests.integration.backend import docker_backend


@unittest.skipIf(os.environ.get('SURVEIL_INTEGRATION_TESTS', None) != 'True',
                 'Skipping integraiton tests')
class MergedIntegrationTest(base.BaseTestCase):

    @classmethod
    def setUpClass(cls):
        test_backend = os.environ.get(
            'SURVEIL_INTEGRATION_TESTS_BACKEND',
            None
        )

        if test_backend == 'DOCKER':
            MergedIntegrationTest.backend = docker_backend.DockerBackend()
        else:
            raise Exception(
                "Could not identify tests backend: '%s'" % test_backend
            )
        cls.backend.setUpClass()

    @classmethod
    def tearDownClass(cls):
        cls.backend.tearDownClass()

    def get_surveil_client(self):
        return MergedIntegrationTest.backend.get_surveil_client()

    def execute_command(self, command, container):
        MergedIntegrationTest.backend.execute_command(command, container)


class SeparatedIntegrationTests(MergedIntegrationTest):

    def setUp(self):
        SeparatedIntegrationTests.setUpClass()

    def tearDown(self):
        SeparatedIntegrationTests.tearDownClass()



# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json

import requests_mock

from surveil.tests.api import functionalTest


class TestEvents(functionalTest.FunctionalTest):

    def setUp(self):
        super(TestEvents, self).setUp()
        self.influxdb_response = json.dumps({
            "results": [
                {
                    "series": [
                        {
                            "name": "ALERT",
                            "tags": {
                                "event_type": "ALERT",
                                "host_name": "myServiceIsDown",
                                "service_description": "iAmADownService"
                            },
                            "columns": [
                                "time",
                                "attempts",
                                "output",
                                "state",
                                "state_type",
                                "alert_type"
                            ],
                            "values": [
                                [
                                    "2015-06-04T18:55:19Z",
                                    1,
                                    "Connection refused",
                                    "CRITICAL",
                                    "SOFT",
                                    "SERVICE"
                                ],
                                [
                                    '2015-06-04T18:55:18Z',
                                    2,
                                    'Connection refused',
                                    'CRITICAL',
                                    'SOFT',
                                    'SERVICE'
                                ],
                                [
                                    '2015-06-04T18:55:17Z',
                                    3,
                                    'Connection refused',
                                    'CRITICAL',
                                    'SOFT',
                                    'SERVICE'
                                ]
                            ]
                        },
                        {
                            'name': 'ALERT',
                            'tags': {
                                'event_type': 'ALERT',
                                'host_name': 'savoirfairelinux',
                                'service_description': 'CPU'
                            },
                            'columns': [
                                'time',
                                'attempts',
                                'output',
                                'state',
                                'state_type',
                                'alert_type'
                            ],
                            'values': [
                                [
                                    '2015-06-04T18:55:16Z',
                                    1,
                                    'Warning - Connection refused',
                                    'CRITICAL',
                                    'HARD',
                                    'SERVICE'
                                ],
                                [
                                    '2015-06-04T18:55:15Z',
                                    2,
                                    'Warning - Connection refused',
                                    'WARNING',
                                    'HARD',
                                    'HOST'
                                ]
                            ]
                        },
                        {
                            'name': 'ALERT',
                            'tags': {
                                'event_type': 'NOTIFICATION',
                                'host_name': 'savoirfairelinux',
                                'service_description': 'CPU'
                            },
                            'columns': [
                                'time',
                                'notification_type',
                                'contact',
                                'state',
                                'notification_method',
                                'acknowledgement'
                            ],
                            'values': [
                                [
                                    '2015-06-04T18:55:14Z',
                                    'SERVICE',
                                    'admin',
                                    'CRITICAL',
                                    'notify-service-by-email',
                                    None
                                ],
                                [
                                    '2015-06-04T18:55:13Z',
                                    'SERVICE',
                                    'admin',
                                    'CRITICAL',
                                    'notify-service-by-email',
                                    'ACKNOWLEDGEMENT'
                                ]
                            ]
                        },
                        {
                            'name': 'ALERT',
                            'tags': {
                                'event_type': 'NOTIFICATION',
                                'host_name': 'Google',
                                'service_description': 'Load'
                            },
                            'columns': [
                                'time',
                                'notification_type',
                                'contact',
                                'state',
                                'notification_method',
                                'acknowledgement'
                            ],
                            'values': [
                                [
                                    '2015-06-04T18:55:12Z',
                                    'SERVICE',
                                    'admin',
                                    'CRITICAL',
                                    'notify-service-by-email',
                                    None
                                ]
                            ]
                        }
                    ]
                }
            ]
        })

    def test_get_all(self):
        expected_values = [
            {
                "host_name": "myServiceIsDown",
                "event_type": "ALERT",
                "service_description": "iAmADownService",
                "time": "2015-06-04T18:55:19Z",
                "attempts": 1,
                "output": "Connection refused",
                "state": "CRITICAL",
                "state_type": "SOFT",
                "alert_type": "SERVICE"
            },
            {
                'host_name': 'myServiceIsDown',
                'event_type': 'ALERT',
                'service_description': 'iAmADownService',
                'time': '2015-06-04T18:55:18Z',
                'attempts': 2,
                'output': 'Connection refused',
                'state': 'CRITICAL',
                'state_type': 'SOFT',
                'alert_type': 'SERVICE'
            },
            {
                'host_name': 'myServiceIsDown',
                'event_type': 'ALERT',
                'service_description': 'iAmADownService',
                'time': '2015-06-04T18:55:17Z',
                'attempts': 3,
                'output': 'Connection refused',
                'state': 'CRITICAL',
                'state_type': 'SOFT',
                'alert_type': 'SERVICE'
            },
            {
                'host_name': 'savoirfairelinux',
                'event_type': 'ALERT',
                'service_description': 'CPU',
                'time': '2015-06-04T18:55:16Z',
                'attempts': 1,
                'output': 'Warning - Connection refused',
                'state': 'CRITICAL',
                'state_type': 'HARD',
                'alert_type': 'SERVICE'
            },
            {
                'host_name': 'savoirfairelinux',
                'event_type': 'ALERT',
                'service_description': 'CPU',
                'time': '2015-06-04T18:55:15Z',
                'attempts': 2,
                'output': 'Warning - Connection refused',
                'state': 'WARNING',
                'state_type': 'HARD',
                'alert_type': 'HOST'
            },
            {
                'host_name': 'savoirfairelinux',
                'event_type': 'NOTIFICATION',
                'service_description': 'CPU',
                'time': '2015-06-04T18:55:14Z',
                'notification_type': 'SERVICE',
                'contact': 'admin',
                'state': 'CRITICAL',
                'notification_method': 'notify-service-by-email'
            },
            {
                'host_name': 'savoirfairelinux',
                'event_type': 'NOTIFICATION',
                'service_description': 'CPU',
                'time': '2015-06-04T18:55:13Z',
                'notification_type': 'SERVICE',
                'contact': 'admin',
                'state': 'CRITICAL',
                'notification_method': 'notify-service-by-email',
                'acknowledgement': 'ACKNOWLEDGEMENT'
            },
            {
                'host_name': 'Google',
                'event_type': 'NOTIFICATION',
                'service_description': 'Load',
                'time': '2015-06-04T18:55:12Z',
                'notification_type': 'SERVICE',
                'contact': 'admin',
                'state': 'CRITICAL',
                'notification_method': 'notify-service-by-email'
            }
        ]

        with requests_mock.Mocker() as m:
            m.register_uri(requests_mock.GET,
                           'http://influxdb:8086/query',
                           text=self.influxdb_response)

            response = self.get('/v2/status/events')

            self.assert_count_equal_backport(
                json.loads(response.body.decode()),
                expected_values
            )

    def test_get_events_for_host(self):
        influxdb_google_response = json.dumps({
            "results": [
                {
                    "series": [
                        {
                            'name': 'ALERT',
                            'tags': {
                                'event_type': 'NOTIFICATION',
                                'host_name': 'Google',
                                'service_description': 'Load'
                            },
                            'columns': [
                                'time',
                                'notification_type',
                                'contact',
                                'state',
                                'notification_method',
                                'acknowledgement'
                            ],
                            'values': [
                                [
                                    '2015-06-04T18:55:12Z',
                                    'SERVICE',
                                    'admin',
                                    'CRITICAL',
                                    'notify-service-by-email',
                                    None
                                ]
                            ]
                        }
                    ]
                }
            ]
        })

        with requests_mock.Mocker() as m:
            m.register_uri(requests_mock.GET,
                           'http://influxdb:8086/query',
                           text=influxdb_google_response)

            query = {
                'filters': json.dumps({
                    "is": {
                        "host_name": ['Google']
                    }
                }),
                'time_interval': {
                    "start_time": "2015-06-04T18:55:02Z",
                    "end_time": "2015-06-04T18:55:42Z"
                }
            }

            response = self.post_json('/v2/status/events', params=query)

            self.assertEqual(
                m.last_request.qs['q'],
                ["select * from event where time >= '2015-06-04t18:55:02z' "
                 "and time <= '2015-06-04t18:55:42z' and host_name='google'"]
            )

            self.assert_count_equal_backport(
                json.loads(response.body.decode()),
                [{
                    'host_name': 'Google',
                    'event_type': 'NOTIFICATION',
                    'service_description': 'Load',
                    'time': '2015-06-04T18:55:12Z',
                    'notification_type': 'SERVICE',
                    'contact': 'admin',
                    'state': 'CRITICAL',
                    'notification_method': 'notify-service-by-email'
                }]
            )

    def test_paging(self):
        with requests_mock.Mocker() as m:
            m.register_uri(requests_mock.GET,
                           'http://influxdb:8086/query',
                           text=self.influxdb_response)

            query = {'paging': {'page': 1, 'size': 2}}

            response = self.post_json('/v2/status/events', params=query)

            self.assertEqual(
                m.last_request.qs['q'],
                ["select * from event limit 4"]
            )

            self.assert_count_equal_backport(
                json.loads(response.body.decode()), [

                    {
                        'host_name': 'myServiceIsDown',
                        'event_type': 'ALERT',
                        'service_description': 'iAmADownService',
                        'time': '2015-06-04T18:55:17Z',
                        'attempts': 3,
                        'output': 'Connection refused',
                        'state': 'CRITICAL',
                        'state_type': 'SOFT',
                        'alert_type': 'SERVICE'
                    },
                    {
                        'host_name': 'savoirfairelinux',
                        'event_type': 'ALERT',
                        'service_description': 'CPU',
                        'time': '2015-06-04T18:55:16Z',
                        'attempts': 1,
                        'output': 'Warning - Connection refused',
                        'state': 'CRITICAL',
                        'state_type': 'HARD',
                        'alert_type': 'SERVICE'
                    }]
            )


# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import mongoengine

from surveil.api.storage.mongodb.config import host
from surveil.api.storage.mongodb import foreign_key_field


class Service(mongoengine.Document):
    meta = {
        'collection': 'services',
        'strict': False
    }
    host_name = foreign_key_field.ForeignKeyListField(host.Host,
                                                      ['host_name', 'name'])
    service_description = mongoengine.StringField()
    contacts = mongoengine.ListField()
    check_command = mongoengine.StringField()
    max_check_attempts = mongoengine.IntField()
    check_interval = mongoengine.IntField()
    retry_interval = mongoengine.IntField()
    check_period = mongoengine.StringField()
    notification_interval = mongoengine.IntField()
    notification_period = mongoengine.StringField()
    contact_groups = mongoengine.ListField()
    passive_checks_enabled = mongoengine.StringField()
    use = mongoengine.ListField()
    name = mongoengine.StringField(unique=True, sparse=True)
    register = mongoengine.StringField()



# Copyright 2015 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import copy
import json


from surveil.api.datamodel.config import contactgroup
from surveil.tests.api import functionalTest


class TestContactGroupsController(functionalTest.FunctionalTest):

    def setUp(self):
        super(TestContactGroupsController, self).setUp()
        self.groups = [
            {
                'contactgroup_name': 'novell-admins',
                'members': ["bob", "alice"],
                'contactgroup_members': []
            },
            {
                'contactgroup_name': 'linux-adminx',
                'members': ['bob'],
                'contactgroup_members': []
            },
        ]
        self.mongoconnection.shinken.contactgroups.insert(
            copy.deepcopy(self.groups)
        )

        self.contacts = [
            {"contact_name": "bob"},
            {"contact_name": "alice"},
        ]
        self.mongoconnection.shinken.contacts.insert(
            copy.deepcopy(self.contacts)
        )

    def test_get_all_contactgroups(self):
        response = self.post_json('/v2/config/contactgroups', params={})

        self.assert_count_equal_backport(
            json.loads(response.body.decode()),
            self.groups
        )
        self.assertEqual(response.status_int, 200)

    def test_get_one_contactgroup(self):
        response = self.get('/v2/config/contactgroups/novell-admins')

        self.assertEqual(
            json.loads(response.body.decode()),
            self.groups[0]
        )

    def test_create_contactgroup(self):
        g = contactgroup.ContactGroup(
            contactgroup_name='John',
            members=["bob", "alice"],
        )

        self.put_json('/v2/config/contactgroups', g.as_dict())

        self.assertIsNotNone(
            self.mongoconnection.shinken.contactgroups.find_one(g.as_dict())
        )

    def test_delete_contactgroup(self):
        self.assertIsNotNone(
            self.mongoconnection.shinken.contactgroups.find_one(self.groups[0])
        )

        self.delete('/v2/config/contactgroups/novell-admins')

        self.assertIsNone(
            self.mongoconnection.shinken.contactgroups.find_one(self.groups[0])
        )

    def test_put_contactgroup(self):
        self.assertEqual(
            self.mongoconnection.shinken.contactgroups.find_one(
                {'contactgroup_name': 'novell-admins'}
            )['members'],
            ['bob', 'alice']
        )

        self.put_json(
            '/v2/config/contactgroups/novell-admins',
            {"contactgroup_name": "novell-admins",
             "members": ["bob"]}
        )

        self.assertEqual(
            self.mongoconnection.shinken.contactgroups.find_one(
                {'contactgroup_name': 'novell-admins'}
            )['members'],
            ['bob']
        )



# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import copy
import json


from surveil.tests.api import functionalTest


class TestBusinessImpactModulationController(functionalTest.FunctionalTest):

    def setUp(self):
        super(TestBusinessImpactModulationController, self).setUp()
        self.modulations = [
            {
                'business_impact_modulation_name': 'LowImpactOnNight',
                'business_impact': 1,
                'modulation_period': 'night'
            },
            {
                'business_impact_modulation_name': 'LowImpactOnDay',
                'business_impact': 1,
                'modulation_period': 'day'
            },
        ]
        self.mongoconnection.shinken.businessimpactmodulations.insert(
            copy.deepcopy(self.modulations)
        )

    def test_get_all_modulations(self):
        response = self.post_json('/v2/config/businessimpactmodulations',
                                  params={})

        self.assert_count_equal_backport(
            json.loads(response.body.decode()),
            [
                {'business_impact': 1,
                 'business_impact_modulation_name': 'LowImpactOnDay',
                 'modulation_period': 'day'},
                {'business_impact': 1,
                 'business_impact_modulation_name': 'LowImpactOnNight',
                 'modulation_period': 'night'},
            ]
        )
        self.assertEqual(response.status_int, 200)

    def test_get_one_modulation(self):
        response = self.get(
            '/v2/config/businessimpactmodulations/LowImpactOnDay'
        )

        self.assertEqual(
            json.loads(response.body.decode()),
            {'business_impact': 1,
             'business_impact_modulation_name': 'LowImpactOnDay',
             'modulation_period': 'day'}
        )

    def test_create_modulation(self):
        m = {'business_impact': 1,
             'business_impact_modulation_name': 'testtt',
             'modulation_period': 'day'}

        self.assertIsNone(
            self.mongoconnection.shinken.businessimpactmodulations.find_one(m)
        )

        self.put_json('/v2/config/businessimpactmodulations', m)

        self.assertIsNotNone(
            self.mongoconnection.shinken.businessimpactmodulations.find_one(m)
        )

    def test_delete_modulation(self):
        self.assertIsNotNone(
            self.mongoconnection.shinken.businessimpactmodulations.find_one(
                {"business_impact_modulation_name": 'LowImpactOnNight'}
            )
        )

        self.delete('/v2/config/businessimpactmodulations/LowImpactOnNight')

        self.assertIsNone(
            self.mongoconnection.shinken.timeperiods.find_one(
                {"business_impact_modulation_name": 'LowImpactOnNight'}
            )
        )

    def test_put_modulation(self):
        self.assertEqual(
            self.mongoconnection.shinken.businessimpactmodulations.find_one(
                {'business_impact_modulation_name': 'LowImpactOnNight'}
            )['modulation_period'],
            'night'
        )

        self.put_json(
            '/v2/config/businessimpactmodulations/LowImpactOnNight',
            {"business_impact_modulation_name": "LowImpactOnNight",
             'business_impact': 1,
             "modulation_period": 'updated'}
        )

        self.assertEqual(
            self.mongoconnection.shinken.businessimpactmodulations.find_one(
                {'business_impact_modulation_name': 'LowImpactOnNight'}
            )['modulation_period'],
            'updated'
        )



# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import pecan
from pecan import rest
import wsme.types as wtypes
import wsmeext.pecan as wsme_pecan

from surveil.api.handlers.bansho import config_handler
from surveil.common import util


class ConfigController(rest.RestController):

    @util.policy_enforce(['authenticated'])
    @wsme_pecan.wsexpose(wtypes.text)
    def get(self):
        """Retrieve user config, empty dict if no config exists."""
        user_name = pecan.request.headers.get('X-User-Id')
        handler = config_handler.ConfigHandler(pecan.request)
        config = handler.get(user_name)
        return config

    @util.policy_enforce(['authenticated'])
    @wsme_pecan.wsexpose(body=wtypes.text,)
    def post(self, config):
        """Save user config.

        :param config: JSON config object
        """
        user_name = pecan.request.headers.get('X-User-Id')
        handler = config_handler.ConfigHandler(pecan.request)
        handler.update(user_name, config)



# Copyright 2015 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import wsme
import wsme.types as wtypes

from surveil.api.datamodel import types


class NotificationWay(types.Base):
    notificationway_name = wsme.wsattr(wtypes.text, mandatory=False)
    host_notification_period = wsme.wsattr(wtypes.text, mandatory=False)
    service_notification_period = wsme.wsattr(wtypes.text, mandatory=False)
    host_notification_options = wsme.wsattr(wtypes.ArrayType(wtypes.text),
                                            mandatory=False)
    service_notification_options = wsme.wsattr(wtypes.ArrayType(wtypes.text),
                                               mandatory=False)
    host_notification_commands = wsme.wsattr(wtypes.ArrayType(wtypes.text),
                                             mandatory=False)
    service_notification_commands = wsme.wsattr(wtypes.ArrayType(wtypes.text),
                                                mandatory=False)
    min_business_impact = wsme.wsattr(int, mandatory=False)

    @classmethod
    def sample(cls):
        return cls(
            notificationway_name="email_in_day",
            host_notification_period="24x7",
            service_notification_period="24x7",
            host_notification_options=["d", "u", "r", "f", "s"],
            service_notification_options=["w", "u", "c", "r", "f"],
            service_notification_commands=["notify-service"],
            host_notification_commands=["notify-host"]
            )



# Copyright 2015 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from surveil.api.handlers import handler


class StatusHandler(handler.Handler):
    """This handler creates MongoDB indexes."""

    def __init__(self, *args, **kwargs):
        super(StatusHandler, self).__init__(*args, **kwargs)

        try:
            self.request.mongo_connection.admin.command(
                'setParameter',
                textSearchEnabled=True
            )
        except Exception:
            pass

        self.request.mongo_connection.alignak_live.hosts.ensure_index(
            [("$**", 'text')]
        )

        self.request.mongo_connection.alignak_live.services.ensure_index(
            [("$**", 'text')]
        )



# Copyright 2015 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json

from surveil.api.datamodel import live_query
from surveil.api.datamodel.status.metrics import time_interval
from surveil.api.handlers.status import influxdb_query
from surveil.tests import base


class LiveQueryFilterTest(base.BaseTestCase):

    def test_build_where_clause(self):
        filters = {
            "is": {
                "state": [0],
                "description": ["test_keystone"]
            }
        }

        result = influxdb_query._build_where_clause(
            filters
        )

        expected = ["WHERE", "description='test_keystone'", "AND", "state=0"]

        self.assertEqual(result, expected)

    def test_build_where_clause_no_filters(self):
        filters = {}

        result = influxdb_query._build_where_clause(
            filters
        )

        expected = ""

        self.assert_count_equal_backport(result, expected)

    def test_build_influx_query(self):
        query = {}
        measurement = 'ALERT'
        group_by = ['*', 'host_name']
        limit = 10

        result = influxdb_query.build_influxdb_query(query,
                                                     measurement,
                                                     group_by=group_by,
                                                     limit=limit)

        expected = "SELECT * FROM ALERT GROUP BY *, host_name LIMIT 10"

        self.assertEqual(expected, result)

    def test_build_influx_query_orderby(self):
        query = live_query.LiveQuery(
            fields=['host_name', 'last_check']
        )
        measurement = 'ALERT'
        group_by = ['*', 'host_name']
        order_by = ['time DESC']
        limit = 10

        result = influxdb_query.build_influxdb_query(query,
                                                     measurement,
                                                     group_by=group_by,
                                                     order_by=order_by,
                                                     limit=limit)

        expected = ("SELECT * FROM ALERT "
                    "GROUP BY *, host_name "
                    "ORDER BY time DESC LIMIT 10")

        self.assertEqual(expected, result)

    def test_build_query_basic(self):
        query = live_query.LiveQuery(
            time_interval=time_interval.TimeInterval(
                start_time="2015-01-29T21:50:44Z",
                end_time="2015-01-29T22:50:44Z"
            )
        )

        group_by = ['host_name', 'service_description']
        order_by = ['time DESC']

        result = influxdb_query.build_influxdb_query(query,
                                                     "metric_pl",
                                                     group_by=group_by,
                                                     order_by=order_by
                                                     )
        expected = ("SELECT * "
                    "FROM metric_pl "
                    "WHERE time >= '2015-01-29T21:50:44Z' "
                    "AND time <= '2015-01-29T22:50:44Z' "
                    "GROUP BY host_name, "
                    "service_description ORDER BY time DESC")

        self.assertEqual(result, expected)

    def test_build_query_host_name(self):
        query = live_query.LiveQuery(
            fields=['host_name'],
            filters=json.dumps({
                "is": {
                    "host_name": ["localhost"]
                }
            }),
            time_interval=time_interval.TimeInterval(
                start_time='2015-01-29T21:50:44Z',
                end_time='2015-01-29T22:50:44Z'
            )
        )
        group_by = ['service_description']
        order_by = ['time DESC']

        result = influxdb_query.build_influxdb_query(query,
                                                     "metric_pl",
                                                     group_by=group_by,
                                                     order_by=order_by
                                                     )
        expected = ("SELECT * "
                    "FROM metric_pl "
                    "WHERE time >= '2015-01-29T21:50:44Z' "
                    "AND time <= '2015-01-29T22:50:44Z' "
                    "AND host_name='localhost' "
                    "GROUP BY service_description "
                    "ORDER BY time DESC")

        self.assertEqual(result, expected)

    def test_build_query_complete(self):
        query = live_query.LiveQuery(
            fields=['host_name'],
            filters=json.dumps({
                "is": {
                    "host_name": ["localhost"],
                    "service_description": ["mySQL"]
                }
            }),
            time_interval=time_interval.TimeInterval(
                start_time='2015-01-29T21:50:44Z',
                end_time='2015-01-29T22:50:44Z'
            )
        )
        order_by = ['time DESC']
        result = influxdb_query.build_influxdb_query(query,
                                                     "metric_pl",
                                                     order_by=order_by
                                                     )

        expected = ("SELECT * "
                    "FROM metric_pl "
                    "WHERE time >= '2015-01-29T21:50:44Z' "
                    "AND time <= '2015-01-29T22:50:44Z' "
                    "AND host_name='localhost' "
                    "AND service_description='mySQL' "
                    "ORDER BY time DESC")

        self.assertEqual(result, expected)



# Copyright 2015 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json

from surveil.api.datamodel import live_query
from surveil.api.handlers.status import fields_filter
from surveil.tests import base


class FieldsFilterTest(base.BaseTestCase):

    def test_filter_fields(self):
        items = [
            {"description": "test_keystone",
             "last_state_change": 1429400986,
             "plugin_output": "OK - 127.0.0.1: rta 0.045ms, lost 0%",
             "last_check": 1429400984, "state": 'OK',
             "host_name": "test_keystone"},
        ]

        query = live_query.LiveQuery(
            fields=['host_name', 'last_check'],
            filters=json.dumps({
                "isnot": {
                    "state": [0, 1],
                    "description": ["test_keystone"]
                }
            })
        )

        result = fields_filter.filter_fields(
            items,
            query
        )

        expected = [{"last_check": 1429400984, "host_name": "test_keystone"}]

        self.assert_count_equal_backport(result, expected)



# Copyright 2010-2012 OpenStack Foundation
# Copyright 2015 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from surveil.api.authmiddleware import utils

import six

"""
Keystone-Compatible Token-based Authentication Middleware.

This Middleware is based on keystonemiddleware, it creates the same headers but
verifies token authenticity against some other service. It was created for
Surveil so that we can have more flexibility on the authentication backend.

"""


_HEADER_TEMPLATE = {
    'X%s-Domain-Id': 'domain_id',
    'X%s-Domain-Name': 'domain_name',
    'X%s-Project-Id': 'project_id',
    'X%s-Project-Name': 'project_name',
    'X%s-Project-Domain-Id': 'project_domain_id',
    'X%s-Project-Domain-Name': 'project_domain_name',
    'X%s-User-Id': 'user_id',
    'X%s-User-Name': 'username',
    'X%s-User-Domain-Id': 'user_domain_id',
    'X%s-User-Domain-Name': 'user_domain_name',
}


def filter_factory(global_conf, **local_conf):
    """Returns a WSGI filter app for use with paste.deploy."""
    conf = global_conf.copy()
    conf.update(local_conf)

    def auth_filter(app):
        return AuthProtocol(app, conf)
    return auth_filter


class AuthProtocol(object):
    """Middleware that handles authenticating client calls."""

    def __init__(self, app, conf):
        self._app = app
        self._init_auth_headers()

        #  TODO(aviau): auth_uri should be loaded in config
        self._auth_uri = 'www.surveil.com'

    def _init_auth_headers(self):
        """Initialize auth header list.

        Both user and service token headers are generated.
        """
        auth_headers = ['X-Service-Catalog',
                        'X-Identity-Status',
                        'X-Service-Identity-Status',
                        'X-Roles',
                        'X-Service-Roles']
        for key in six.iterkeys(_HEADER_TEMPLATE):
            auth_headers.append(key % '')
            # Service headers
            auth_headers.append(key % '-Service')

        self._auth_headers = auth_headers

    def __call__(self, env, start_response):
        """Handle incoming request.

        Authenticate send downstream on success. Reject request if
        we can't authenticate.

        """
        self._remove_auth_headers(env)

        token = self._get_header(env, 'X-Auth-Token', None)

        #  TODO(aviau): Validate token, then build proper headers
        if token == "aaaaa-bbbbb-ccccc-dddd":
            user_headers = {
                'X-Identity-Status': 'Confirmed',
                'X-User-Id': 'surveil-default-user',
                'X-Roles': 'admin,surveil',
                'X-Service-Catalog': 'surveil'
            }
            self._add_headers(env, user_headers)

            service_headers = {
                'X-Service-Identity-Status': 'Confirmed',
                'X-Service-Roles': 'surveil',
            }
            self._add_headers(env, service_headers)

        return self._call_app(env, start_response)

    def _remove_auth_headers(self, env):
        """Remove headers so a user can't fake authentication.

        Both user and service token headers are removed.

        :param env: wsgi request environment

        """
        self._remove_headers(env, self._auth_headers)

    def _remove_headers(self, env, keys):
        """Remove http headers from environment."""
        for k in keys:
            env_key = self._header_to_env_var(k)
            try:
                del env[env_key]
            except KeyError:
                pass

    def _add_headers(self, env, headers):
        """Add http headers to environment."""
        for (k, v) in six.iteritems(headers):
            env_key = self._header_to_env_var(k)
            env[env_key] = v

    def _header_to_env_var(self, key):
        """Convert header to wsgi env variable.

        :param key: http header name (ex. 'X-Auth-Token')
        :returns: wsgi env variable name (ex. 'HTTP_X_AUTH_TOKEN')

        """
        return 'HTTP_%s' % key.replace('-', '_').upper()

    def _get_header(self, env, key, default=None):
        """Get http header from environment."""
        env_key = self._header_to_env_var(key)
        return env.get(env_key, default)

    def _call_app(self, env, start_response):
        # NOTE(jamielennox): We wrap the given start response so that if an
        # application with a 'delay_auth_decision' setting fails, or otherwise
        # raises Unauthorized that we include the Authentication URL headers.
        def _fake_start_response(status, response_headers, exc_info=None):
            if status.startswith('401'):
                response_headers.extend(self._reject_auth_headers)
            return start_response(status, response_headers, exc_info)
        return self._app(env, _fake_start_response)

    def _reject_request(self, env, start_response):
        """Redirect client to auth server.

        :param env: wsgi request environment
        :param start_response: wsgi response callback
        :returns: HTTPUnauthorized http response
        """
        resp = utils.MiniResp('Authentication required',
                              env, self._reject_auth_headers)
        start_response('401 Unauthorized', resp.headers)
        return resp.body

    @property
    def _reject_auth_headers(self):
        header_val = 'Keystone uri=\'%s\'' % self._auth_uri
        return [('WWW-Authenticate', header_val)]



# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from surveil.api.datamodel.status.metrics import metric as m
from surveil.api.handlers import handler


class MetricNameHandler(handler.Handler):
    """Fulfills a request on the metrics."""

    def get(self, host_name, service_description=None):
        """Return all metrics name."""
        service_description = service_description or '__host__'
        query = ("SHOW measurements WHERE host_name='%s' "
                 "AND service_description='%s'"
                 % (host_name, service_description))
        influx_client = self.request.influxdb_client
        response = influx_client.query(query)

        metrics = []
        for item in response[None]:
            metric_dict = self._metrics_name_from_influx_item(item)
            if metric_dict is not None:
                metrics.append(m.Metric(**metric_dict))

        return metrics

    def _metrics_name_from_influx_item(self, item):
        metric_name = None
        mappings = [('metric_name', 'name', str), ]
        for field in mappings:
            value = item.get(field[1], None)
            if value is not None:
                if value.startswith('metric_'):
                    metric_name = {}
                    metric_name[field[0]] = field[2](value[7:])

        return metric_name



# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import os

import influxdb
import mongoengine
import mongomock
from oslo_config import cfg
import pecan
from pecan import hooks
import pecan.testing
import pymongo

from surveil.tests import base


__all__ = ['FunctionalTest']


class FunctionalTest(base.BaseTestCase):
    """Used for functional tests.

    Used where you need to test your literal
    application and its integration with the framework.
    """

    def setUp(self):

        if os.environ.get('SURVEIL_FUNCTIONAL_MONGOMOCK', None) == 'True':
            self.mongoconnection = mongomock.Connection()
        else:
            self.mongoconnection = pymongo.MongoClient()
            self.mongoconnection.drop_database('shinken')
            self.mongoconnection.drop_database('alignak_live')
            self.mongoconnection.drop_database('surveil')

        self.ws_arbiter_url = "http://localhost:7760"
        self.influxdb_client = influxdb.InfluxDBClient.from_DSN(
            'influxdb://root:root@influxdb:8086/db'
        )

        class TestHook(hooks.PecanHook):
            def __init__(self, mongoclient, wsarbiterurl, influxdb_client):
                self.mongoclient = mongoclient
                self.ws_arbiter_url = wsarbiterurl
                self.influxdb_client = influxdb_client

            def before(self, state):
                state.request.mongo_connection = self.mongoclient
                state.request.ws_arbiter_url = self.ws_arbiter_url
                state.request.influxdb_client = self.influxdb_client

                def get_connection(alias):
                    return self.mongoclient

                mongoengine.connection.get_connection = get_connection
                mongoengine.connect('shinken')

        app_hooks = [
            TestHook(
                self.mongoconnection,
                self.ws_arbiter_url,
                self.influxdb_client
            )
        ]

        policy_path = os.path.dirname(os.path.realpath(__file__))

        opts = [
            cfg.StrOpt('config_dir', default=policy_path),
            cfg.StrOpt('config_file', default='policy.json'),
            cfg.StrOpt('project', default='surveil'),
        ]

        cfg.CONF.register_opts(opts)

        self.app = pecan.testing.load_test_app({
            'app': {
                'root': 'surveil.api.controllers.root.RootController',
                'modules': ['surveil.api'],
                'debug': True,
                'hooks': app_hooks
            }
        })

        self.auth_headers = {
            'X-Identity-Status': 'Confirmed',
            'X-User-Id': 'surveil-default-user',
            'X-Roles': 'admin,surveil',
            'X-Service-Catalog': 'surveil',
            'X-Service-Identity-Status': 'Confirmed',
            'X-Service-Roles': 'surveil',
        }

        def make_action(verb):
            target = getattr(self.app, verb)

            def func(*args, **kwargs):
                kwargs.setdefault('headers', self.auth_headers)
                return target(*args, **kwargs)
            return func

        for action in ('get', 'post', 'put', 'delete',
                       'post', 'post_json', 'put_json',
                       'delete_json'):
            setattr(self, action, make_action(action))

    def tearDown(self):
        if os.environ.get('SURVEIL_FUNCTIONAL_MONGOMOCK', None) != 'True':
            self.mongoconnection.close()
        pecan.set_config({}, overwrite=True)



# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from surveil.api.handlers.config import mongoengine_query
from surveil.api.handlers import handler


class MongoObjectHandler(handler.Handler):
    """Fulfills a request on a MongoDB resource."""

    def __init__(self,
                 resource_datamodel,
                 resource_storage,
                 *args,
                 **kwargs):
        super(MongoObjectHandler, self).__init__(*args, **kwargs)
        self.resource_datamodel = resource_datamodel
        self.resource_storage = resource_storage

    def _get_mongoengine_object(self, identifier):
        return self.resource_storage.objects.get(**identifier)

    def _get_dict(self, mongoengine_object):
        json_object = mongoengine_object.to_mongo().to_dict()
        json_object.pop('_id', None)
        return json_object

    def get(self, identifier):
        """Return the resource."""
        mongoengine_object = self._get_mongoengine_object(identifier)
        resource_dict = self._get_dict(mongoengine_object)
        return self.resource_datamodel(**resource_dict)

    def update(self, identifier, resource):
        """Modify an existing resource."""
        r = self._get_mongoengine_object(identifier)
        resource_dict = resource.as_dict()
        for key, value in resource_dict.items():
            setattr(r, key, value)
        r.save()

    def delete(self, identifier):
        """Delete existing resource."""
        r = self._get_mongoengine_object(identifier)
        r.delete()
        r.save()

    def create(self, resource):
        """Create a new resource."""
        r = self.resource_storage(**resource.as_dict())
        r.save()

    def get_all(self, lq={}):
        """Return all resources."""

        fields, query, skip, limit = mongoengine_query.build_mongoengine_query(
            lq, self.resource_storage
        )

        if skip is not None and limit is not None:
            objects = (
                self.resource_storage.objects
                .filter(query)
                .only(*fields)
                .skip(skip)
                .limit(limit)
            )
        else:
            objects = self.resource_storage.objects.filter(query).only(*fields)

        resp = [
            self.resource_datamodel(**self._get_dict(r))
            for r
            in objects
        ]

        # Mongoengine's 'only()' does not seem to work :(
        filtered_response = []
        if fields:
            for obj in resp:
                obj_with_field = {}
                for field in fields:
                    obj_with_field[field] = getattr(obj, field)
                filtered_response.append(
                    self.resource_datamodel(**obj_with_field)
                )

        return filtered_response or resp



# Copyright 2015 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json

from surveil.api.datamodel import live_query
from surveil.api.datamodel.status import paging
from surveil.api.handlers.status import mongodb_query
from surveil.tests import base


class MongoDBQueryTest(base.BaseTestCase):

    def test_build_mongo_query(self):
        query = live_query.LiveQuery(
            fields=['host_name', 'last_check'],
            filters=json.dumps({
                "isnot": {
                    "state": [0, 1],
                    "last_check": ["test_keystone"]
                }
            }),
            paging=paging.Paging(size=7, page=4)
        )

        service_mappings = {
            "last_check": "last_chk",
            "description": "service_description",
            "plugin_output": "output",
            "acknowledged": "problem_has_been_acknowledged",
        }
        lq = mongodb_query.translate_live_query(
            query.as_dict(),
            service_mappings
        )

        self.assertEqual(
            lq,
            {'fields': [u'host_name', 'last_chk'],
             'filters': {u'isnot': {u'state': [0, 1],
                                    'last_chk': [u'test_keystone']}},
             'paging': query.paging},
        )

        query, kwargs = mongodb_query.build_mongodb_query(lq)

        expected_query = {
            "state": {"$nin": [0, 1]},
            "last_chk": {"$nin": ["test_keystone"]}
        }

        expected_fields = {
            "host_name": 1,
            "last_chk": 1
        }

        self.assertEqual(query[0], expected_query)
        self.assertEqual(query[1], expected_fields)
        self.assertEqual(kwargs, {'limit': 7, 'skip': 28})



# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from surveil.api.datamodel.config import contact as datamodel
from surveil.api.handlers import mongodb_mongoengine_object_handler
from surveil.api.storage.mongodb.config import contact as storage


class ContactHandler(mongodb_mongoengine_object_handler.MongoObjectHandler):
    """Fulfills a request on the Contact resource."""

    def __init__(self, *args, **kwargs):
        super(ContactHandler, self).__init__(
            datamodel.Contact,
            storage.Contact,
            *args,
            **kwargs
        )



# Copyright 2015 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import pecan
from pecan import rest
import wsmeext.pecan as wsme_pecan

from surveil.api.datamodel import live_query
from surveil.api.datamodel.status.metrics import metric as m
from surveil.api.handlers.status.metrics import metric_handler
from surveil.api.handlers.status.metrics import metric_name_handler
from surveil.common import util


class MetricsController(rest.RestController):

    @util.policy_enforce(['authenticated'])
    @wsme_pecan.wsexpose([m.Metric])
    def get(self):
        """Returns all metrics name for a host."""
        handler = metric_name_handler.MetricNameHandler(pecan.request)
        metrics_name = handler.get(pecan.request.context['host_name'])
        return metrics_name

    @pecan.expose()
    def _lookup(self, metric_name, *remainder):
        return MetricController(metric_name), remainder


class MetricController(rest.RestController):

    def __init__(self, metric_name):
        pecan.request.context['metric_name'] = metric_name
        self.metric_name = metric_name

    @util.policy_enforce(['authenticated'])
    @wsme_pecan.wsexpose(m.Metric)
    def get(self):
        """Return the last measure for the metric name on the host."""
        handler = metric_handler.MetricHandler(pecan.request)
        metric = handler.get(
            pecan.request.context['host_name'],
            self.metric_name
        )
        return metric

    @util.policy_enforce(['authenticated'])
    @wsme_pecan.wsexpose([m.Metric], body=live_query.LiveQuery)
    def post(self, query):
        """Given a live query, returns all matching metrics.

        :param live_query: a live query within the request body.
        """
        handler = metric_handler.MetricHandler(pecan.request)
        metrics = handler.get_all(query=query,
                                  metric_name=self.metric_name,
                                  host_name=pecan.request.context['host_name']
                                  )
        return metrics



# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import pecan

from surveil.api import hooks

# Server Specific Configurations
server = {
    'port': 5311,
    'host': '0.0.0.0'
}

authentication = {
    'config_file': 'policy.json',
}


app_hooks = [
    hooks.DBHook(
        pecan.conf.surveil_api_config['mongodb_uri'],
        pecan.conf.surveil_api_config['ws_arbiter_url'],
        pecan.conf.surveil_api_config['influxdb_uri']
    )
]

# Pecan Application Configurations
app = {
    'root': 'surveil.api.controllers.root.RootController',
    'modules': ['surveil.api'],
    'template_path': '%(confdir)s/pecanrest/templates',
    'debug': True,
    'errors': {
        404: '/error/404',
        '__force_dict__': True
    },
    'hooks': app_hooks,
}

logging = {
    'loggers': {
        'root': {'level': 'INFO', 'handlers': ['console']},
        'pecanrest': {'level': 'DEBUG', 'handlers': ['console']},
        'pecan.commands.serve': {'level': 'DEBUG', 'handlers': ['console']},
        'py.warnings': {'handlers': ['console']},
        '__force_dict__': True
    },
    'handlers': {
        'console': {
            'level': 'DEBUG',
            'class': 'logging.StreamHandler',
            'formatter': 'color'
        }
    },
    'formatters': {
        'simple': {
            'format': ('%(asctime)s %(levelname)-5.5s [%(name)s]'
                       '[%(threadName)s] %(message)s')
        },
        'color': {
            '()': 'pecan.log.ColorFormatter',
            'format': ('%(asctime)s [%(padded_color_levelname)s] [%(name)s]'
                       '[%(threadName)s] %(message)s'),
            '__force_dict__': True
        }
    }
}



# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import wsme
import wsme.types as wtypes

from surveil.api.datamodel import types


class TimeInterval(types.Base):
    """Hold a time."""

    start_time = wsme.wsattr(wtypes.text, mandatory=True)
    "The starting time."

    end_time = wsme.wsattr(wtypes.text, mandatory=True)
    "The ending time."

    @classmethod
    def sample(cls):
        return cls(
            start_time='2015-01-29T21:50:44Z',
            end_time='2015-01-29T22:50:44Z'
        )



# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""Script to push a Shinken pack to Surveil"""

import argparse
import sys

import surveilclient.client as sc

from surveil.cmd import surveil_from_nagios


def main():
    # Parse the arguments
    parser = argparse.ArgumentParser(
        prog='surveil-pack-upload',
        add_help=False,
    )
    parser.add_argument('--surveil_api_url',
                        default='http://localhost:5311/v2',
                        type=str)
    parser.add_argument('--surveil_auth_url',
                        default='http://localhost:5311/v2/auth',
                        type=str)
    parser.add_argument('--surveil_api_version',
                        default='2_0',
                        type=str)
    parser.add_argument('pack',
                        metavar='[Pack]',
                        type=str,
                        nargs=1,
                        help='Pack directory')

    (options, args) = parser.parse_known_args(sys.argv[1:])

    cli_surveil = sc.Client(options.surveil_api_url,
                            auth_url=options.surveil_auth_url,
                            version=options.surveil_api_version)

    upload_pack(options.pack[0], cli_surveil)


def upload_pack(pack_dir, client):
    # pack_name = os.path.basename(os.path.normpath(pack_dir))

    surveil_config = surveil_from_nagios.load_config(pack_dir)

    config_manager = client.config
    for object_type, objects in surveil_config:
        object_manager = getattr(config_manager, object_type)
        for object in objects:
            object_manager.create(**object)



# Copyright 2015 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from surveil.api.handlers import handler


class ConfigHandler(handler.Handler):
    """Fulfills a request on the bansho config ressource."""

    def get(self, user_name):
        """Returns user config, empty dict by default."""
        user_name = str(user_name)
        c = self.request.mongo_connection.surveil.bansho.config.find_one(
            {"user_name": user_name}, {'_id': 0}
        )
        config = (c or {}).get('config', {})
        return config

    def update(self, user_name, config):
        """Modify user config."""
        user_name = str(user_name)

        config_dict = {
            "user_name": user_name,
            "config": config
        }

        c = self.request.mongo_connection.surveil.bansho.config.find_one(
            {"user_name": user_name}, {'_id': 0}
        )

        if c is None:
            self.request.mongo_connection.surveil.bansho.config.insert(
                config_dict
            )
        else:
            self.request.mongo_connection.surveil.bansho.config.update(
                {"user_name": user_name},
                config_dict
            )



# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import pecan
from pecan import rest
import wsmeext.pecan as wsme_pecan

from surveil.api.controllers.v2.status.hosts import metrics as host_metrics
from surveil.api.controllers.v2.status.hosts import results as host_results
from surveil.api.controllers.v2.status.hosts.services import (
    services as host_services)
from surveil.api.datamodel import live_query
from surveil.api.datamodel.status import live_host
from surveil.api.handlers.status import live_host_handler
from surveil.common import util


class HostsController(rest.RestController):

    @util.policy_enforce(['authenticated'])
    @wsme_pecan.wsexpose([live_host.LiveHost])
    def get_all(self):
        """Returns all hosts."""
        handler = live_host_handler.HostHandler(pecan.request)
        hosts = handler.get_all()
        return hosts

    @util.policy_enforce(['authenticated'])
    @wsme_pecan.wsexpose([live_host.LiveHost], body=live_query.LiveQuery)
    def post(self, query):
        """Given a LiveQuery, returns all matching hosts."""
        handler = live_host_handler.HostHandler(pecan.request)
        hosts = handler.get_all(live_query=query)
        return hosts

    @pecan.expose()
    def _lookup(self, host_name, *remainder):
        return HostController(host_name), remainder


class HostController(rest.RestController):

    # See init for controller creation. We need host_name to instanciate it
    # externalcommands = ExternalCommandsController()
    # config = config.ConfigController()
    services = host_services.ServicesController()
    metrics = host_metrics.MetricsController()
    results = host_results.CheckResultsSubController()

    def __init__(self, host_name):
        pecan.request.context['host_name'] = host_name
        self.host_name = host_name

    @util.policy_enforce(['authenticated'])
    @wsme_pecan.wsexpose(live_host.LiveHost)
    def get(self):
        """Returns a specific host."""
        handler = live_host_handler.HostHandler(pecan.request)
        host = handler.get(self.host_name)
        return host


class ConfigController(rest.RestController):

    @util.policy_enforce(['authenticated'])
    @pecan.expose()
    def get_all(self):
        """Returns config from a specific host."""
        return "Dump CONFIG"



# Copyright 2015 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import mongoengine


def validate_refering_object_exists(referring_object, field_name, value):
    return referring_object.objects(**{field_name: value}).count() > 0


class ForeignKeyStringField(mongoengine.StringField):

    def __init__(self, referring_object,
                 possible_field_names,
                 *args,
                 **kwargs):
        super(ForeignKeyStringField, self).__init__(*args, **kwargs)
        self.refering_object = referring_object
        self.possible_field_names = possible_field_names

    def validate(self, value):
        mongoengine.StringField.validate(self, value)

        for field in self.possible_field_names:
            if validate_refering_object_exists(self.refering_object,
                                               field,
                                               value) is True:
                return
        self.error(
            "Could not find matching %s" % value
        )


class ForeignKeyListField(mongoengine.ListField):
    def __init__(self, referring_object,
                 possible_field_names,
                 *args,
                 **kwargs):
        super(ForeignKeyListField, self).__init__(*args, **kwargs)
        self.refering_object = referring_object
        self.possible_field_names = possible_field_names

    def validate(self, value):
        mongoengine.ListField.validate(self, value)

        for key in value:
            found_matching = False

            for field in self.possible_field_names:
                if validate_refering_object_exists(
                        self.refering_object,
                        field,
                        key) is True:
                    found_matching = True
                    break

            if found_matching is False:
                self.error(
                    "Could not find matching %s" % key
                )



# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import pecan
from pecan import rest

from surveil.common import util


class HelloController(rest.RestController):

    @pecan.expose()
    @util.policy_enforce(['pass'])
    def get(self):
        """Says hello."""
        return "Hello World!"

    @pecan.expose()
    def _lookup(self, *remainder):
        return HelloSubController(), remainder


class AdminController(rest.RestController):

    @pecan.expose()
    @util.policy_enforce(['admin'])
    def get(self):
        """Says hello to the admin."""
        return "Hello, dear admin!"


class DeniedController(rest.RestController):

    @pecan.expose()
    @util.policy_enforce(['break'])
    def get(self):
        """This should be denied."""
        return "Looks like policies are not working."


class HelloSubController(rest.RestController):
    admin = AdminController()
    denied = DeniedController()



# Copyright 2015 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import copy
import json

from surveil.tests.api import functionalTest


class TestStatusServices(functionalTest.FunctionalTest):

    def setUp(self):
        super(TestStatusServices, self).setUp()
        self.services = [
            {
                "host_name": 'Webserver',
                "service_description": 'Apache',
                "description": 'Serves Stuff',
                "state": 'OK',
                "last_chk": 1429220785,
                "last_state_change": 1429220785,
                "plugin_output": 'HTTP OK - GOT NICE RESPONSE',
                "long_output": 'The response\nwas really\nnice',
                "problem_has_been_acknowledged": True,
            },
            {
                "host_name": 'someserver',
                "service_description": 'servicesomething',
                "description": 'Serves  lots of Stuff',
                "state": 'UNKNOWN',
                "last_chk": 1429220785,
                "last_state_change": 1429220785,
                "plugin_output": 'Hi there',
                "long_output": 'I am;\nthe servicessomthing;\noutput;',
                "problem_has_been_acknowledged": False,
            },
        ]
        self.mongoconnection.alignak_live.services.insert(
            copy.deepcopy(self.services)
        )

    def test_get_all_services(self):
        response = self.get("/v2/status/services")

        expected = [
            {
                "host_name": 'Webserver',
                "service_description": 'Apache',
                "description": 'Serves Stuff',
                "state": 'OK',
                "last_check": 1429220785,
                "last_state_change": 1429220785,
                "plugin_output": 'HTTP OK - GOT NICE RESPONSE',
                "long_output": 'The response\nwas really\nnice',
                'acknowledged': True,
            },
            {
                "host_name": 'someserver',
                "service_description": 'servicesomething',
                "description": 'Serves  lots of Stuff',
                "state": 'UNKNOWN',
                "last_check": 1429220785,
                "last_state_change": 1429220785,
                "plugin_output": 'Hi there',
                "long_output": 'I am;\nthe servicessomthing;\noutput;',
                'acknowledged': False,
            },
        ]

        self.assertEqual(json.loads(response.body.decode()),
                         expected)

    def test_query_services(self):
        query = {
            'fields': ['host_name', 'service_description'],
            'filters': json.dumps({
                "isnot": {
                    "host_name": ['ws-arbiter'],
                },
                "is": {
                    "service_description": ["Apache"],
                    "acknowledged": [True],
                }
            })
        }

        response = self.post_json("/v2/status/services", params=query)

        expected = [
            {'host_name': 'Webserver',
             'service_description': 'Apache'}
        ]

        self.assertEqual(json.loads(response.body.decode()),
                         expected)



# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from __future__ import print_function

import argparse
import collections
import fnmatch
import json
import os
import re
import sys

import six
import wsme

from surveil.api.datamodel.config import businessimpactmodulation
from surveil.api.datamodel.config import checkmodulation
from surveil.api.datamodel.config import command
from surveil.api.datamodel.config import contact
from surveil.api.datamodel.config import contactgroup
from surveil.api.datamodel.config import host
from surveil.api.datamodel.config import hostgroup
from surveil.api.datamodel.config import macromodulation
from surveil.api.datamodel.config import notificationway
from surveil.api.datamodel.config import realm
from surveil.api.datamodel.config import service
from surveil.api.datamodel.config import servicegroup
from surveil.api.datamodel.config import timeperiod


def main():
    # Parse the arguments
    parser = argparse.ArgumentParser(add_help=False)

    parser.add_argument(
        'path',
        metavar='[path]',
        type=str,
        nargs=1,
        help='Configuration path'
    )

    (options, args) = parser.parse_known_args(sys.argv[1:])

    pack_dir = options.path[0]

    surveil_config = load_config(pack_dir)

    print(json.dumps(surveil_config, indent=4, sort_keys=True))


def load_config(path):
    """From a directory, returns Surveil configuration"""

    if os.path.isdir(path):
        # Find the .cfg files
        cfg_files = [
            os.path.join(dirpath, f)
            for dirpath, dirnames, files in os.walk(path)
            for f in fnmatch.filter(files, '*.cfg')
        ]
    else:
        cfg_files = [path]

    nagios_config = collections.OrderedDict()

    for cfg_file in sorted(cfg_files):
        # Open the file
        f = open(cfg_file, 'r')
        config_string = f.read()
        f.close()

        # Load the config
        file_config = _load_nagios_config(config_string)

        # Append to the loaded config
        for object_type, objects in file_config.items():
            nagios_config[object_type] = nagios_config.get(
                object_type,
                []
            ) + objects

    surveil_config = _transform_config(nagios_config)

    sorted_surveil_config = _sort_config(surveil_config)

    return sorted_surveil_config


def _load_nagios_config(config_string):
    """Given a nagios configuration string, returns a python dict"""
    config = collections.OrderedDict()

    # Find all config objects
    config_objects = re.finditer(
        r'define\s(?P<object_type>\w*)\s*{(?P<object_properties>[^{}]*)}',
        config_string
    )

    # For each object in the file...
    for object_match in config_objects:
        object_type = object_match.group("object_type")

        config_object = {}

        # For each property of the object...
        for property_match in re.finditer(
                r'(?P<property>[^\s]+)\s*(?P<value>.*?)\s*\n',
                object_match.group("object_properties")
        ):

            config_object[
                property_match.group('property')
            ] = property_match.group('value')

        # Append the config object
        config[object_type + 's'] = config.get(
            object_type + 's', []
        ) + [config_object]

    return config


def _transform_config(nagios_config):
    """Given a nagios config dict, returns surveil configuration"""
    transformed_config = collections.OrderedDict()

    for object_type, objects in nagios_config.items():
        for config_object in objects:

            # PROPERTY NAMES TRANSFORMATIONS
            name_transformed_obj = _transform_property_names(
                config_object,
                object_type
            )

            # PROPERTY TYPES TRANSORMATIONS
            type_transformed_obj = _transform_property_types(
                name_transformed_obj,
                object_type
            )

            transformed_config[object_type] = transformed_config.get(
                object_type,
                []
            ) + [type_transformed_obj]

    return transformed_config


def _transform_property_types(config_object, object_type):
    transformed_object = collections.OrderedDict()

    datamodels = {
        "businessimpactmodulations": businessimpactmodulation.BusinessImpactModulation,
        "checkmodulations": checkmodulation.CheckModulation,
        "commands": command.Command,
        "contacts": contact.Contact,
        "contactgroups": contactgroup.ContactGroup,
        "hosts": host.Host,
        "hostgroups": hostgroup.HostGroup,
        "macromodulations": macromodulation.MacroModulation,
        "notificationways": notificationway.NotificationWay,
        "realms": realm.Realm,
        "services": service.Service,
        "servicegroups": servicegroup.ServiceGroup,
        "timeperiods": timeperiod.TimePeriod,
    }
    object_datamodel = datamodels[object_type]
    object_wsmeattrs = object_datamodel._wsme_attributes
    for attribute in object_wsmeattrs:

        if attribute.name in config_object:
            # COMMA-SEPARATED => List
            if isinstance(attribute._get_datatype(), wsme.types.ArrayType):
                transformed_object[attribute.name] = config_object[attribute.name].split(',')
            # Integers
            elif attribute._get_datatype() in six.integer_types:
                transformed_object[attribute.name] = int(config_object[attribute.name])
            # Strings
            else:
                transformed_object[attribute.name] = config_object[attribute.name]

    return transformed_object


def _transform_property_names(config_object, object_type):
    transformed_object = collections.OrderedDict()

    # HOSTS
    if object_type in ['hosts', 'services']:
        transformed_object['custom_fields'] = {}
        for property, value in config_object.items():
            if property.startswith('_'):
                transformed_object['custom_fields'][property] = value
            else:
                transformed_object[property] = value
    # TIMEPERIODS
    elif object_type == 'timeperiods':
        transformed_object['periods'] = {}
        properties = [p.name for p in timeperiod.TimePeriod._wsme_attributes]
        for property, value in config_object.items():
            if property not in properties:
                transformed_object['periods'][property] = value
            else:
                transformed_object[property] = value
    # OTHER OBJECTS
    else:
        for property, value in config_object.items():
            transformed_object[property] = value
    return transformed_object


def _sort_config(surveil_config):

    # Sort object types
    correct_order = {
        "realms": 0,
        "timeperiods": 1,
        "macromodulations": 2,
        "commands": 3,
        "checkmodulations": 4,
        "businessimpactmodulations": 5,
        "notificationways": 6,
        "contacts": 7,
        "contactgroups": 8,
        "hosts": 9,
        "services": 10,
        "hostgroups": 11,
        "servicegroups": 12,
    }
    sorted_object_types = sorted(surveil_config.items(),
                                 key=lambda x: correct_order.get(x[0], 99))

    sorted_config = []

    # Sort objects
    for item in sorted_object_types:
        object_type = item[0]
        objects = item[1]

        if object_type in ['hosts', 'services']:
            objects = _sort_objects(objects)

        sorted_config.append((object_type, objects))

    return sorted_config


def _sort_objects(objects):
    sorted_objects = []
    while len(objects) > 0:
        for object in objects:
            host_dependencies = object.get('use', [])
            unsolved_dependencies = [
                d for d in host_dependencies
                if not any(o.get('name', None) == d
                           for o in sorted_objects)
            ]

            if len(unsolved_dependencies) == 0:
                break

        sorted_objects.append(object)
        objects.remove(object)

    return sorted_objects


if __name__ == "__main__":
    main()



# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""Starter script for the Surveil API service."""

from __future__ import print_function
from __future__ import unicode_literals
import argparse
import os
import sys

from oslo_config import cfg
from pecan.commands import base

from surveil.api import app

CONF = cfg.CONF

OPTS = [
    cfg.StrOpt(
        'api_paste_config',
        default="api_paste.ini",
        help="Configuration file for WSGI definition of API."
    ),
]

CONF.register_opts(OPTS)


class SurveilCommandRunner(base.CommandRunner):

    def __init__(self):
        super(SurveilCommandRunner, self).__init__()
        self.parser = argparse.ArgumentParser(description='Surveil API server')
        self.parser.add_argument('--reload', '-r', action='store_true',
                                 help='Automatically reload as code changes')
        self.parser.add_argument('--pecan_config', '-p',
                                 default='/etc/surveil/config.py',
                                 help='Pecan config file (config.py)')
        self.parser.add_argument('--api_paste_config', '-a',
                                 default='/etc/surveil/api_paste.ini',
                                 help='API Paste config file (api_paste.ini)')
        self.parser.add_argument('--config_file', '-c',
                                 default='/etc/surveil/surveil.cfg',
                                 help='Pecan config file (surveil.cfg)')

    def run(self, args):
        namespace = self.parser.parse_args(args)
        # Get absolute paths
        namespace.pecan_config = os.path.join(os.getcwd(),
                                              namespace.pecan_config)
        namespace.api_paste_config = os.path.join(os.getcwd(),
                                                  namespace.api_paste_config)
        namespace.config_file = os.path.join(os.getcwd(),
                                             namespace.config_file)

        # Check conf files exist
        if not os.path.isfile(namespace.pecan_config):
            print("Bad config file: %s" % namespace.pecan_config,
                  file=sys.stderr)
            sys.exit(1)
        if not os.path.isfile(namespace.api_paste_config):
            print("Bad config file: %s" % namespace.api_paste_config,
                  file=sys.stderr)
            sys.exit(2)
        if not os.path.isfile(namespace.config_file):
            print("Bad config file: %s" % namespace.config_file,
                  file=sys.stderr)
            sys.exit(1)

        app.SurveilCommand().run(namespace)

    @classmethod
    def handle_command_line(cls):  # pragma: nocover
        runner = SurveilCommandRunner()
        runner.run(sys.argv[1:])



# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import pecan
from pecan import rest
import wsmeext.pecan as wsme_pecan

from surveil.api.datamodel.config import service
from surveil.api.datamodel import live_query as lq
from surveil.api.handlers.config import service_handler
from surveil.common import util


class ServicesController(rest.RestController):

    @util.policy_enforce(['authenticated'])
    @wsme_pecan.wsexpose([service.Service], body=lq.LiveQuery)
    def post(self, data):
        """Returns all services."""
        handler = service_handler.ServiceHandler(pecan.request)
        services = handler.get_all(data)
        return services

    @util.policy_enforce(['authenticated'])
    @wsme_pecan.wsexpose(service.Service,
                         body=service.Service,
                         status_code=201)
    def put(self, data):
        """Create a new service.

        :param data: a service within the request body.
        """
        handler = service_handler.ServiceHandler(pecan.request)
        services = handler.create(data)
        return services



# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import copy
import json

from surveil.api.datamodel.config import service
from surveil.tests.api import functionalTest


class TestServiceController(functionalTest.FunctionalTest):

    def setUp(self):
        super(TestServiceController, self).setUp()
        self.services = [
            {
                "host_name": ["sample-server1"],
                "service_description": "check-",
                "check_command": "check-disk!/dev/sdb1",
                "max_check_attempts": 5,
                "check_interval": 5,
                "retry_interval": 3,
                "check_period": "24x7",
                "notification_interval": 30,
                "notification_period": "24x7",
                "contacts": ["surveil-ptl", "surveil-bob"],
                "contact_groups": ["linux-masters"],
                "use": []
            },
            {
                "host_name": ["sample-server2"],
                "service_description": "check-disk-sdb2",
                "check_command": "check-disk!/dev/sdb1",
                "max_check_attempts": 5,
                "check_interval": 5,
                "retry_interval": 3,
                "check_period": "24x7",
                "notification_interval": 30,
                "notification_period": "24x7",
                "contacts": ["surveil-ptl", "surveil-bob"],
                "contact_groups": ["linux-admins"],
                "use": []
            },
            {
                "host_name": ["sample-server3"],
                "service_description": "check-disk-sdb3",
                "check_command": "check-disk!/dev/sdb1",
                "max_check_attempts": 5,
                "check_interval": 5,
                "retry_interval": 3,
                "check_period": "24x7",
                "notification_interval": 30,
                "notification_period": "24x7",
                "contacts": ["surveil-ptl", "surveil-bob"],
                "contact_groups": ["linux-admins"],
                "use": []
            },
        ]
        self.mongoconnection.shinken.services.insert(
            copy.deepcopy(self.services)
        )

        self.hosts = [
            {"host_name": "sample-server1"}
        ]
        self.mongoconnection.shinken.hosts.insert(
            copy.deepcopy(self.services)
        )

    def test_get_all_services(self):
        response = self.post_json('/v2/config/services', params={})

        self.assert_count_equal_backport(
            json.loads(response.body.decode()),
            self.services
        )
        self.assertEqual(response.status_int, 200)

    def test_search(self):
        response = self.post_json('/v2/config/services', params={
            "search": "admins",
            "fields": ['service_description']
        })

        self.assert_count_equal_backport(
            json.loads(response.body.decode()),
            [
                {'service_description': 'check-disk-sdb2'},
                {'service_description': 'check-disk-sdb3'},
            ]
        )

    def test_get_all_services_templates(self):
        self.mongoconnection.shinken.services.insert(
            copy.deepcopy(
                {"host_name": ["sample-server444"],
                 "service_description": "check-disk-sdb2",
                 "check_command": "check-disk!/dev/sdb1",
                 "max_check_attempts": 5,
                 "check_interval": 5,
                 "retry_interval": 3,
                 "check_period": "24x7",
                 "notification_interval": 30,
                 "notification_period": "24x7",
                 "contacts": ["surveil-ptl", "surveil-bob"],
                 "name": "Template",
                 "register": "0",
                 "contact_groups": ["linux-admins"],
                 "use": []}
            )
        )

        post_lq = {"filters": '{"is":{"register": "0"},'
                              '"defined":{"name": "True"}}'}
        response = self.post_json('/v2/config/services', params=post_lq)

        self.assert_count_equal_backport(
            json.loads(response.body.decode()),
            [{"host_name": ["sample-server444"],
              "service_description": "check-disk-sdb2",
              "check_command": "check-disk!/dev/sdb1",
              "max_check_attempts": 5,
              "check_interval": 5,
              "retry_interval": 3,
              "check_period": "24x7",
              "notification_interval": 30,
              "notification_period": "24x7",
              "contacts": ["surveil-ptl", "surveil-bob"],
              "name": "Template",
              "register": "0",
              "contact_groups": ["linux-admins"],
              "use":[]}]
        )

        response = self.post_json('/v2/config/services', params=post_lq)

        self.assertEqual(
            len(json.loads(response.body.decode())),
            1
        )

        self.assertEqual(response.status_int, 200)

    def test_add_service(self):
        new_service = {
            "host_name": ["sample-server1"],
            "service_description": "check-new-thing",
            "check_command": "check-disk!/dev/sdb1",
            "max_check_attempts": 5,
            "check_interval": 5,
            "retry_interval": 3,
            "check_period": "24x7",
            "notification_interval": 30,
            "notification_period": "24x7",
            "contacts": ["surveil-ptl", "surveil-bob"],
            "contact_groups": ["linux-admins"],
            "use": []
        }
        response = self.put_json(
            "/v2/config/services",
            params=new_service
        )

        services = [service.Service(**s).as_dict() for s in
                    self.mongoconnection.shinken.services.find()]

        self.assertTrue(new_service in services)
        self.assertEqual(response.status_int, 201)



# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from surveil.api.datamodel.status import live_host
from surveil.api.handlers.status import mongodb_query
from surveil.api.handlers.status import status_handler


class HostHandler(status_handler.StatusHandler):
    """Fulfills a request on the live hosts."""

    def get(self, host_name):
        """Return a host."""
        mongo_s = self.request.mongo_connection.alignak_live.hosts.find_one(
            {"host_name": host_name}
        )

        return live_host.LiveHost(**_host_dict_from_mongo_item(mongo_s))

    def get_all(self, live_query=None):
        """Return all live hosts."""

        host_mappings = {
            "last_check": "last_chk",
            "description": "display_name",
            "plugin_output": "output",
            "acknowledged": "problem_has_been_acknowledged"
        }

        if live_query:
            lq = mongodb_query.translate_live_query(live_query.as_dict(),
                                                    host_mappings)
        else:
            lq = {}

        query, kwargs = mongodb_query.build_mongodb_query(lq)

        mongo_dicts = (self.request.mongo_connection.
                       alignak_live.hosts.find(*query, **kwargs))

        host_dicts = [
            _host_dict_from_mongo_item(s) for s in mongo_dicts
        ]

        hosts = []
        for host_dict in host_dicts:
            host = live_host.LiveHost(**host_dict)
            hosts.append(host)

        return hosts


def _host_dict_from_mongo_item(mongo_item):
    """Create a dict from a mongodb item."""

    mappings = [
        ('last_chk', 'last_check', int),
        ('last_state_change', 'last_state_change', int),
        ('output', 'plugin_output', str),
        ('problem_has_been_acknowledged', 'acknowledged', bool),
        ('state', 'state', str),
        ('display_name', 'description', str),
    ]

    for field in mappings:
        value = mongo_item.pop(field[0], None)
        if value is not None:
            mongo_item[field[1]] = field[2](value)

    return mongo_item



# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from surveil.api.datamodel import live_query as q
from surveil.api.datamodel.status.metrics import metric as m
from surveil.api.handlers import handler
from surveil.api.handlers.status import influxdb_query


class MetricHandler(handler.Handler):
    """Fulfills a request on the metrics."""

    def get(self, host_name, metric_name, service_description=None):
        """Return the last metric."""
        query = self._build_metric_query(
            host_name,
            metric_name,
            service_description=service_description,
            limit=1)

        influx_client = self.request.influxdb_client
        response = influx_client.query(query)

        metrics = []
        for item in response[None]:
            metric_dict = self._metric_dict_from_influx_item(item, metric_name)
            metric = m.Metric(**metric_dict)
            metrics.append(metric)

        if metric_name:
            metrics = metrics[0] or ''
        return metrics

    def get_all(self, metric_name,
                host_name, service_description=None,
                query=q.LiveQuery()):
        """Return all metrics."""
        query = self._build_metric_query(
            host_name,
            metric_name,
            service_description=service_description,
            query=query)
        influx_client = self.request.influxdb_client
        response = influx_client.query(query)

        metrics = []
        for item in response[None]:
            metric_dict = self._metric_dict_from_influx_item(item, metric_name)
            metric = m.Metric(**metric_dict)
            metrics.append(metric)

        return metrics

    def _build_metric_query(self, host_name, metric_name,
                            service_description=None,
                            query=None, limit=None):
        filters = {
            "is": {
                "host_name": [host_name]
            }
        }

        group_by = []
        if service_description:
            filters["is"]["service_description"] = [service_description]
        else:
            group_by.append('service_description')

        return influxdb_query.build_influxdb_query(query,
                                                   'metric_' + metric_name,
                                                   order_by=["time desc"],
                                                   group_by=group_by,
                                                   additional_filters=filters,
                                                   limit=limit,
                                                   )

    def _metric_dict_from_influx_item(self, item, metric_name=None):
        metric_dict = {"metric_name": str(metric_name)}
        mappings = [
            ('min', str),
            ('max', str),
            ('critical', str),
            ('warning', str),
            ('value', str),
            ('unit', str),
        ]

        for field in mappings:
            value = item.get(field[0], None)
            if value is not None:
                if field[0] == 'name':
                    if value.startswith('metric_'):
                        metric_dict = {}
                        metric_dict[field[1]] = field[2](value[7:])
                else:
                    metric_dict[field[0]] = field[1](value)

        return metric_dict



# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import wsme


def filter_fields(item_list, live_query):
    """Takes unwanted keys out of a dict depending on a live_query."""
    filtered_items = []

    if live_query.fields != wsme.Unset:
        fields = live_query.fields
        for item in item_list:
            filtered_item = {}
            for field in fields:
                filtered_item[field] = item.get(field, None)
            filtered_items.append(filtered_item)
    else:
        filtered_items = item_list

    return filtered_items



# Copyright 2015 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import copy
import json
import unittest

import requests_mock
from six.moves import urllib_parse

from surveil.tests.api import functionalTest


class TestStatusHosts(functionalTest.FunctionalTest):

    def setUp(self):
        super(TestStatusHosts, self).setUp()
        self.host = [
            {
                "display_name": "localhost",
                "address": "127.0.0.1",
                "childs": [],
                "parents": ['parent.com'],
                "last_chk": 1.429405764e+09,
                "last_state_change": 1.429405765316929e+09,
                "plugin_output": "OK - localhost: rta 0.033ms, lost 0%",
                "long_output": "This is\na\nlong\noutput",
                "state": "OK",
                "state_type": "HARD",
                "problem_has_been_acknowledged": True,
                "host_name": "localhost",
                "services": ["load", "cpu"],
            },
            {
                "display_name": "test_keystone",
                "address": "127.0.0.1",
                "childs": [],
                "parents": ['parent.com'],
                "last_chk": 1.429405763e+09,
                "last_state_change": 1.429405765317144e+09,
                "plugin_output":  "OK - 127.0.0.1: rta 0.032ms, lost 0%",
                "long_output": "Another very\nlong\noutput",
                "state": "OK",
                "state_type": "HARD",
                "problem_has_been_acknowledged": True,
                "host_name": "test_keystone",
                "services": ["apache"],
            },
            {
                "display_name": "ws-arbiter",
                "address": "127.0.0.1",
                "childs": ['test_keystone'],
                "parents": ['parent.com'],
                "last_chk": 1.429405764e+09,
                "last_state_change": 1.429405765317063e+09,
                "plugin_output": "OK - localhost: rta 0.030ms, lost 0%",
                "long_output": "What a;\nlong;\noutput;",
                "state": "OK",
                "state_type": "HARD",
                "problem_has_been_acknowledged": True,
                "host_name": "ws-arbiter",
                "services": [],
            },
        ]
        self.mongoconnection.alignak_live.hosts.insert(
            copy.deepcopy(self.host)
        )

        self.services = [
            {
                "host_name": 'Webserver',
                "service_description": 'Apache',
                "description": 'Serves Stuff',
                "state": 'OK',
                "last_check": 1429220785,
                "last_state_change": 1429220785,
                "plugin_output": 'HTTP OK - GOT NICE RESPONSE',
                "long_output": 'This has\n a long\noutput'
            },
            {
                "host_name": 'someserver',
                "service_description": 'servicesomething',
                "description": 'Serves  lots of Stuff',
                "state": 'OK',
                "last_check": 1429220785,
                "last_state_change": 1429220785,
                "plugin_output": 'Hi there',
                "long_output": 'My output;\nis better;\nthan yours;'
            },

        ]
        self.mongoconnection.alignak_live.services.insert(
            copy.deepcopy(self.services)
        )

    def test_get_all_hosts(self):
        response = self.get("/v2/status/hosts")

        expected = [
            {"description": "localhost",
             "address": "127.0.0.1",
             "childs": [],
             "parents": ['parent.com'],
             "last_state_change": 1429405765,
             "plugin_output": "OK - localhost: rta 0.033ms, lost 0%",
             "long_output": "This is\na\nlong\noutput",
             "last_check": 1429405764,
             "state": "OK",
             "acknowledged": True,
             "services": ["load", "cpu"],
             "host_name": "localhost"},
            {"description": "test_keystone",
             "address": "127.0.0.1",
             "childs": [],
             "parents": ['parent.com'],
             "last_state_change": 1429405765,
             "plugin_output": "OK - 127.0.0.1: rta 0.032ms, lost 0%",
             "long_output": "Another very\nlong\noutput",
             "last_check": 1429405763,
             "state": "OK",
             "acknowledged": True,
             "services": ["apache"],
             "host_name": "test_keystone"},
            {"description": "ws-arbiter",
             "address": "127.0.0.1",
             "childs": ['test_keystone'],
             "parents": ['parent.com'],
             "last_state_change": 1429405765,
             "plugin_output": "OK - localhost: rta 0.030ms, lost 0%",
             "long_output": "What a;\nlong;\noutput;",
             "last_check": 1429405764,
             "state": "OK",
             "acknowledged": True,
             "services": [],
             "host_name": "ws-arbiter"}]

        self.assert_count_equal_backport(json.loads(response.body.decode()),
                                         expected)

    def test_query_hosts(self):
        query = {
            'fields': ['host_name', 'last_check'],
            'filters': json.dumps({
                "isnot": {
                    "host_name": ['localhost'],
                    "description": ['test_keystone']
                }
            })
        }

        response = self.post_json("/v2/status/hosts", params=query)

        expected = [{"host_name": "ws-arbiter", "last_check": 1429405764}]

        self.assert_count_equal_backport(json.loads(response.body.decode()),
                                         expected)

    @unittest.skip("Does not work on jenkins")
    def test_search_hosts(self):
        query = {
            'fields': ['host_name'],
            'search': 'another'
        }

        response = self.post_json("/v2/status/hosts", params=query)

        self.assertEqual(
            json.loads(response.body.decode()),
            [{"host_name": "test_keystone"}]
        )

    def test_query_host_paging(self):
        query = {
            'paging': {
                'page': 1,
                'size': 2
            }
        }

        response = self.post_json("/v2/status/hosts", params=query)

        self.assertEqual(
            json.loads(response.body.decode()),
            [
                {'acknowledged': True,
                 'address': u'127.0.0.1',
                 'childs': [u'test_keystone'],
                 'description': u'ws-arbiter',
                 'host_name': u'ws-arbiter',
                 'last_check': 1429405764,
                 'last_state_change': 1429405765,
                 'long_output': u'What a;\nlong;\noutput;',
                 'parents': [u'parent.com'],
                 'plugin_output': u'OK - localhost: rta 0.030ms, lost 0%',
                 'services': [],
                 'state': u'OK'}
            ]
        )

    def test_get_specific_host(self):

        response = self.get("/v2/status/hosts/localhost")

        expected = {"childs": ["test_keystone"],
                    "parents": ['parent.com'],
                    "description": "localhost",
                    "last_state_change": 1429812192,
                    "acknowledged": True,
                    "plugin_output": "OK - localhost: rta 0.044ms, lost 0%",
                    "long_output": "This is\na\nlong\noutput",
                    "last_check": 1429812191,
                    "state": "OK",
                    "host_name": "localhost",
                    "services": ["load", "cpu"],
                    "address": "localhost"}

        self.assert_count_equal_backport(json.loads(response.body.decode()),
                                         expected)

    def test_get_specific_host_service(self):
        response = self.get(
            "/v2/status/hosts/someserver/services/servicesomething"
        )
        expected = {'description': 'check-ws-arbiter',
                    'last_state_change': 1429823532,
                    'plugin_output': ('TCP OK - 0.000 second '
                                      'response time on port 7760'),
                    "long_output": 'My output;\nis better;\nthan yours;',
                    'last_check': 1429823531,
                    'state': 'OK',
                    'host_name': 'ws-arbiter',
                    'service_description': 'check-ws-arbiter'}
        self.assert_count_equal_backport(json.loads(response.body.decode()),
                                         expected)

    def test_submit_service_result(self):
        with requests_mock.Mocker() as m:
            m.register_uri(requests_mock.POST,
                           self.ws_arbiter_url + "/push_check_result")

            check_result = {
                "return_code": "0",
                "output": "TEST OUTPUT",
                "time_stamp": "1409149234"
            }

            response = self.post_json(
                "/v2/status/hosts/bogus-router/services/" +
                "service-example/results",
                params=check_result
            )

            self.assertEqual(response.status_int, 204)
            self.assertEqual(
                urllib_parse.parse_qs(m.last_request.body),
                {
                    u'output': [u'TEST OUTPUT'],
                    u'return_code': [u'0'],
                    u'service_description': [u'service-example'],
                    u'host_name': [u'bogus-router'],
                    u'time_stamp': [u'1409149234']
                }
            )

    def test_submit_host_result(self):
        with requests_mock.Mocker() as m:
            m.register_uri(requests_mock.POST,
                           self.ws_arbiter_url + "/push_check_result")

            check_result = {
                "return_code": "0",
                "output": "TEST OUTPUT",
                "time_stamp": "1409149234"
            }

            response = self.post_json("/v2/status/hosts/bogus-router/results",
                                      params=check_result)

            self.assertEqual(response.status_int, 204)
            self.assertEqual(
                urllib_parse.parse_qs(m.last_request.body),
                {
                    u'output': [u'TEST OUTPUT'],
                    u'return_code': [u'0'],
                    u'host_name': [u'bogus-router'],
                    u'time_stamp': [u'1409149234']
                }
            )



# Copyright 2015 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import pecan
from pecan import rest
import wsmeext.pecan as wsme_pecan

from surveil.api.datamodel.config import businessimpactmodulation as mod
from surveil.api.datamodel import live_query as lq
from surveil.api.handlers.config import businessimpactmodulation_handler as bh
from surveil.common import util


class BusinessImpactModulationsController(rest.RestController):

    @pecan.expose()
    def _lookup(self, businessimpactmodulation_name, *remainder):
        return BusinessImpactModulationController(
            businessimpactmodulation_name), remainder

    @util.policy_enforce(['authenticated'])
    @wsme_pecan.wsexpose([mod.BusinessImpactModulation], body=lq.LiveQuery)
    def post(self, data):
        """Returns all business impact modulations."""
        handler = bh.BusinessImpactModulationHandler(pecan.request)
        modulations = handler.get_all(data)
        return modulations

    @util.policy_enforce(['authenticated'])
    @wsme_pecan.wsexpose(mod.BusinessImpactModulation,
                         body=mod.BusinessImpactModulation,
                         status_code=201)
    def put(self, data):
        """Create a new business impact modulation.

        :param data: a business impact modulation within the request body.
        """
        handler = bh.BusinessImpactModulationHandler(pecan.request)
        handler.create(data)


class BusinessImpactModulationController(rest.RestController):

    def __init__(self, business_impact_modulation_name):
        pecan.request.context['business_impact_modulation_name'] = (
            business_impact_modulation_name)
        self._id = business_impact_modulation_name

    @util.policy_enforce(['authenticated'])
    @wsme_pecan.wsexpose(mod.BusinessImpactModulation)
    def get(self):
        """Returns a specific business impact modulation."""
        handler = bh.BusinessImpactModulationHandler(pecan.request)
        modulation = handler.get(
            {"business_impact_modulation_name": self._id}
        )
        return modulation

    @util.policy_enforce(['authenticated'])
    @wsme_pecan.wsexpose(None, status_code=204)
    def delete(self):
        """Delete this business impact modulation."""
        handler = bh.BusinessImpactModulationHandler(pecan.request)
        handler.delete(
            {"business_impact_modulation_name": self._id}
        )

    @util.policy_enforce(['authenticated'])
    @wsme_pecan.wsexpose(None,
                         body=mod.BusinessImpactModulation,
                         status_code=204)
    def put(self, modulation):
        """Update a specific business impact modulation."""
        handler = bh.BusinessImpactModulationHandler(pecan.request)
        handler.update(
            {"business_impact_modulation_name": self._id},
            modulation
        )



# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import copy
import json


from surveil.tests.api import functionalTest


class TestTimePeriodsController(functionalTest.FunctionalTest):

    def setUp(self):
        super(TestTimePeriodsController, self).setUp()
        self.timeperiods = [
            {
                'timeperiod_name': 'nonworkhours',
                "periods": {
                    'sunday': '00:00-24:00',
                    'monday': '00:00-09:00,17:00-24:00'
                }
            },
            {
                'timeperiod_name': 'misc-single-days',
                "periods": {
                    '1999-01-28': '00:00-24:00',
                    'day 2': '00:00-24:00',
                }
            },
        ]
        self.mongoconnection.shinken.timeperiods.insert(
            copy.deepcopy(self.timeperiods)
        )

    def test_get_all_timeperiods(self):
        response = self.post_json('/v2/config/timeperiods', params={})

        self.assert_count_equal_backport(
            json.loads(response.body.decode()),
            [
                {'exclude': [],
                 'timeperiod_name': 'misc-single-days',
                 'periods': {'day 2': '00:00-24:00',
                             '1999-01-28': '00:00-24:00'}},
                {'exclude': [],
                 'timeperiod_name': 'nonworkhours',
                 'periods': {'sunday': '00:00-24:00',
                             'monday': '00:00-09:00,17:00-24:00'}}
            ]
        )
        self.assertEqual(response.status_int, 200)

    def test_get_one_timeperiod(self):
        response = self.get('/v2/config/timeperiods/nonworkhours')

        self.assertEqual(
            json.loads(response.body.decode()),
            {'exclude': [],
             'timeperiod_name': 'nonworkhours',
             'periods': {'sunday': '00:00-24:00',
                         'monday': '00:00-09:00,17:00-24:00'}}
        )

    def test_create_timeperiod(self):
        t = {"timeperiod_name": 'someperiod',
             "periods": {
                 "monday": "fun day",
                 "tuesday": "pizza day"
             }}

        self.put_json('/v2/config/timeperiods', t)

        self.assertIsNotNone(
            self.mongoconnection.shinken.timeperiods.find_one(
                {"timeperiod_name": 'someperiod'}
            )
        )

    def test_delete_timeperiod(self):
        self.assertIsNotNone(
            self.mongoconnection.shinken.timeperiods.find_one(
                {"timeperiod_name": 'nonworkhours'}
            )
        )

        self.delete('/v2/config/timeperiods/nonworkhours')

        self.assertIsNone(
            self.mongoconnection.shinken.timeperiods.find_one(
                {"timeperiod_name": 'nonworkhours'}
            )
        )

    def test_put_timeperiod(self):
        self.assertEqual(
            self.mongoconnection.shinken.timeperiods.find_one(
                {'timeperiod_name': 'nonworkhours'}
            )['periods']['sunday'],
            '00:00-24:00'
        )

        self.put_json(
            '/v2/config/timeperiods/nonworkhours',
            {"timeperiod_name": "nonworkhours",
             "periods": {"sunday": "updated"}}
        )

        self.assertEqual(
            self.mongoconnection.shinken.timeperiods.find_one(
                {'timeperiod_name': 'nonworkhours'}
            )['periods']['sunday'],
            'updated'
        )



# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import pecan
from pecan import rest
import wsmeext.pecan as wsme_pecan

from surveil.api.datamodel.config import host
from surveil.api.datamodel.config import service
from surveil.api.datamodel import live_query as lq
from surveil.api.handlers.config import host_handler
from surveil.api.handlers.config import service_handler
from surveil.common import util


class HostServiceSubController(rest.RestController):

    def __init__(self, service_description):
        pecan.request.context['service_description'] = service_description
        self._id = service_description

    @util.policy_enforce(['authenticated'])
    @wsme_pecan.wsexpose(service.Service)
    def get(self):
        """Returns a specific service."""
        handler = service_handler.ServiceHandler(pecan.request)
        s = handler.get(
            {
                "host_name": pecan.request.context['host_name'],
                "service_description": pecan.request.context[
                    'service_description'
                ]
            }
        )
        return s

    @util.policy_enforce(['authenticated'])
    @wsme_pecan.wsexpose(None, status_code=204)
    def delete(self):
        """Delete a specific service."""
        handler = service_handler.ServiceHandler(pecan.request)
        handler.delete(
            {
                "host_name": pecan.request.context['host_name'],
                "service_description": pecan.request.context[
                    'service_description'
                ]
            }
        )


class HostServicesSubController(rest.RestController):

    @util.policy_enforce(['authenticated'])
    @wsme_pecan.wsexpose([service.Service])
    def get_all(self):
        """Returns all services assocaited with this host."""
        handler = service_handler.ServiceHandler(pecan.request)
        services = handler.get_all(lq.LiveQuery(
            filters='{"is":{"host_name": ["%s"]}}'
                    % pecan.request.context['host_name']))
        return services

    @pecan.expose()
    def _lookup(self, service_description, *remainder):
        return HostServiceSubController(service_description), remainder


class HostSubController(rest.RestController):
    services = HostServicesSubController()


class HostController(rest.RestController):

    def __init__(self, host_name):
        pecan.request.context['host_name'] = host_name
        self._id = host_name

    @util.policy_enforce(['authenticated'])
    @wsme_pecan.wsexpose(host.Host)
    def get(self):
        """Returns a specific host."""
        handler = host_handler.HostHandler(pecan.request)
        h = handler.get({"host_name": self._id})
        return h

    @util.policy_enforce(['authenticated'])
    @wsme_pecan.wsexpose(None, body=host.Host, status_code=204)
    def put(self, data):
        """Modify this host.

        :param data: a host within the request body.
        """
        handler = host_handler.HostHandler(pecan.request)
        handler.update({"host_name": self._id}, data)

    @util.policy_enforce(['authenticated'])
    @wsme_pecan.wsexpose(None, status_code=204)
    def delete(self):
        """Delete this host."""
        handler = host_handler.HostHandler(pecan.request)
        handler.delete({"host_name": self._id})

    @pecan.expose()
    def _lookup(self, *remainder):
        return HostSubController(), remainder


class HostsController(rest.RestController):

    @pecan.expose()
    def _lookup(self, host_name, *remainder):
        return HostController(host_name), remainder

    @util.policy_enforce(['authenticated'])
    @wsme_pecan.wsexpose([host.Host], body=lq.LiveQuery)
    def post(self, data):
        """Returns all hosts."""
        handler = host_handler.HostHandler(pecan.request)
        hosts = handler.get_all(data)
        return hosts

    @util.policy_enforce(['authenticated'])
    @wsme_pecan.wsexpose(host.Host, body=host.Host, status_code=201)
    def put(self, data):
        """Create a new host.

        :param data: a host within the request body.
        """
        handler = host_handler.HostHandler(pecan.request)
        handler.create(data)



# Copyright 2015 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import pecan
from pecan import rest
import wsme.types as wtypes
import wsmeext.pecan as wsme_pecan

from surveil.api.datamodel.config import contact
from surveil.api.datamodel import live_query as lq
from surveil.api.handlers.config import contact_handler
from surveil.common import util


class ContactsController(rest.RestController):

    @pecan.expose()
    def _lookup(self, contact_name, *remainder):
        return ContactController(contact_name), remainder

    @util.policy_enforce(['authenticated'])
    @wsme_pecan.wsexpose([contact.Contact], body=lq.LiveQuery)
    def post(self, data):
        """Returns all contacts."""
        handler = contact_handler.ContactHandler(pecan.request)
        hosts = handler.get_all(data)
        return hosts

    @util.policy_enforce(['authenticated'])
    @wsme_pecan.wsexpose(body=contact.Contact, status_code=201)
    def put(self, data):
        """Create a new contact.

        :param data: a contact within the request body.
        """
        handler = contact_handler.ContactHandler(pecan.request)
        handler.create(data)


class ContactController(rest.RestController):

    def __init__(self, contact_name):
        pecan.request.context['contact_name'] = contact_name
        self._id = contact_name

    @util.policy_enforce(['authenticated'])
    @wsme_pecan.wsexpose(None, status_code=204)
    def delete(self):
        """Returns a specific contact."""
        handler = contact_handler.ContactHandler(pecan.request)
        handler.delete({"contact_name": self._id})

    @util.policy_enforce(['authenticated'])
    @wsme_pecan.wsexpose(None,
                         body=contact.Contact,
                         status_code=204)
    def put(self, contact):
        """Returns a specific contact."""
        handler = contact_handler.ContactHandler(pecan.request)
        handler.update({"contact_name": self._id}, contact)

    @util.policy_enforce(['authenticated'])
    @wsme_pecan.wsexpose(contact.Contact, wtypes.text)
    def get(self):
        """Returns a specific contact."""
        handler = contact_handler.ContactHandler(pecan.request)
        contact = handler.get({"contact_name": self._id})
        return contact


# Copyright 2015 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json

import requests_mock

from surveil.tests.api import functionalTest


class TestHostMetric(functionalTest.FunctionalTest):

    def setUp(self):
        super(TestHostMetric, self).setUp()
        self.influxdb_response = json.dumps({
            "results": [
                {
                    "series": [
                        {"name": "metric_load1",
                         "tags": {"host_name": "srv-monitoring-01",
                                  "service_description": "load",
                                  },
                         "columns": [
                             "time",
                             "critical",
                             "min",
                             "value",
                             "warning"
                         ],
                         "values": [
                             ["2015-04-19T01:09:24Z",
                              "30",
                              "0",
                              "0.60",
                              "15"]
                         ]},
                        {"name": "metric_load1",
                         "tags": {"host_name": "test_keystone",
                                  "service_description": "unload"},
                         "columns": [
                             "time",
                             "critical",
                             "min",
                             "value",
                             "warning"
                         ],
                         "values": [
                             ["2015-04-19T01:09:23Z",
                              "60",
                              "0",
                              "1.5",
                              "20"]
                         ]},
                        {"name": "metric_load1",
                         "tags": {"host_name": "ws-arbiter",
                                  },
                         "columns": [
                             "time",
                             "critical",
                             "min",
                             "value",
                             "warning"
                         ],
                         "values": [
                             ["2015-04-19T01:09:24Z",
                              "20",
                              "0",
                              "6",
                              "10"],
                         ]}
                    ]
                }
            ]
        })

    def test_get(self):
        """Test get all metric names for a service."""
        influxdb_response = json.dumps({
            "results": [
                {
                    "series": [
                        {
                            "name": "measurements",
                            "columns": ["name"],
                            "values": [
                                    ["metric_rtmin"],
                                    ["ALERT"]
                            ]
                        }
                    ]
                }
            ]
        })
        with requests_mock.Mocker() as m:
            m.register_uri(requests_mock.GET,
                           "http://influxdb:8086/query",
                           text=influxdb_response)

            response = self.get(
                "/v2/status/hosts/localhost/services/load/metrics"
            )

            expected = [{"metric_name": "rtmin"}, ]

            self.assert_count_equal_backport(
                json.loads(response.body.decode()),
                expected)
            self.assertEqual(
                m.last_request.qs['q'],
                ["show measurements where host_name='localhost' "
                 "and service_description='load'"]
            )

    def test_get_metric_name(self):
        """Test get the last measure for a metric within a service."""
        self.influxdb_response = json.dumps({
            "results": [
                {
                    "series": [
                        {
                            "name": "measurements",
                            "columns": ["name"],
                            "values": [
                                ["metric_rtmin"],
                                ["ALERT"]
                            ]
                        }
                    ]
                }
            ]
        })
        with requests_mock.Mocker() as m:
            m.register_uri(requests_mock.GET,
                           "http://influxdb:8086/query",
                           text=self.influxdb_response)

            response = self.get(
                "/v2/status/hosts/ws-arbiter/services/load/metrics"
            )

            expected = [{"metric_name": "rtmin"}, ]

            self.assert_count_equal_backport(
                json.loads(response.body.decode()),
                expected)
            self.assertEqual(
                m.last_request.qs['q'],
                ["show measurements where host_name='ws-arbiter' "
                 "and service_description='load'"]
            )

    def test_post_live_query(self):
        """Test posting a live query."""
        self.influxdb_response = json.dumps({
            "results": [
                {
                    "series": [
                        {"name": "metric_load1",
                         "tags": {"host_name": "srv-monitoring-01",
                                  "service_description": "load"},
                         "columns": ["time",
                                     "critical",
                                     "min",
                                     "value",
                                     "warning",
                                     ],
                         "values": [["2015-04-19T01:09:24Z",
                                     "30",
                                     "0",
                                     "0.6",
                                     "15"],
                                    ["2015-04-19T01:09:25Z",
                                     "40",
                                     "4",
                                     "10",
                                     "10"]]}]}]

        })

        with requests_mock.Mocker() as m:
            m.register_uri(requests_mock.GET,
                           "http://influxdb:8086/query",
                           text=self.influxdb_response)

            query = {
                'fields': [],
                'time_interval': {
                    'start_time': '2015-04-19T00:09:24Z',
                    'end_time': '2015-04-19T02:09:25Z'
                },
                'paging': {
                    'size': 10,
                    'page': 3
                }
            }

            response = self.post_json("/v2/status/hosts/srv-monitoring-01/"
                                      "services/load/metrics/load1",
                                      params=query)

            expected = [{"metric_name": 'load1',
                         "min": "0",
                         "critical": "30",
                         "warning": "15",
                         "value": "0.6"
                         },
                        {"metric_name": 'load1',
                         "min": "4",
                         "critical": "40",
                         "warning": "10",
                         "value": "10"
                         }]

            self.assertEqual(
                m.last_request.qs['q'],
                ["select * from metric_load1 "
                 "where time >= '2015-04-19t00:09:24z' "
                 "and time <= '2015-04-19t02:09:25z' "
                 "and host_name='srv-monitoring-01' "
                 "and service_description='load' "
                 "order by time desc "
                 "limit 10 offset 40"
                 ]
            )
            self.assert_count_equal_backport(
                json.loads(response.body.decode()),
                expected)



# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from __future__ import print_function
from __future__ import unicode_literals

import subprocess
import sys
import threading
import time

from paste import deploy
import pecan
from pecan import commands
from pecan import configuration
from six.moves import configparser


global_pecan_config_file = None


def setup_app(pecan_config):
    app_conf = dict(pecan_config.app)

    app = pecan.make_app(
        app_conf.pop('root'),
        logging=getattr(pecan_config, 'logging', {}),
        **app_conf
    )

    return app


def app_factory(global_config, **local_conf):
    global global_pecan_config_file
    return VersionSelectorApplication(global_pecan_config_file)


class VersionSelectorApplication(object):
    def __init__(self, pecan_conf_file):
        pc = pecan.configuration.conf_from_file(pecan_conf_file)

        self.v1 = setup_app(pecan_config=pc)
        self.v2 = setup_app(pecan_config=pc)
        self.config = pc

    def __call__(self, environ, start_response):
        if environ['PATH_INFO'].startswith('/v1/'):
            return self.v1(environ, start_response)
        return self.v2(environ, start_response)


class SurveilCommand(commands.ServeCommand):

    def run(self, args):
        global global_pecan_config_file
        global_pecan_config_file = args.pecan_config
        super(commands.ServeCommand, self).run(args)
        app = self.load_app()
        self.args = args
        self.serve(app, app.config)

    def load_app(self):
        config = configparser.ConfigParser()
        config.read(self.args.config_file)
        surveil_cfg = {"surveil_api_config":
                       dict([i for i in config.items("surveil")])}
        configuration.set_config(surveil_cfg, overwrite=True)
        configuration.set_config(self.args.pecan_config, overwrite=False)

        app = deploy.loadapp('config:%s' % self.args.api_paste_config)
        app.config = configuration._runtime_conf
        return app

    def create_subprocess(self):
        self.server_process = subprocess.Popen(
            [arg for arg in sys.argv if arg not in ['--reload', '-r']],
            stdout=sys.stdout, stderr=sys.stderr
        )

    # TODO(future useless): delete this function when
    # https://review.openstack.org/#/c/206213/
    # will be released
    def watch_and_spawn(self, conf):
        import watchdog.events as events
        import watchdog.observers as observers

        print('Monitoring for changes...',
              file=sys.stderr)

        self.create_subprocess()
        parent = self

        class AggressiveEventHandler(events.FileSystemEventHandler):

            def __init__(self):
                self.wait = False

            def should_reload(self, event):
                for t in (
                    events.FileSystemMovedEvent,
                    events.FileModifiedEvent,
                    events.DirModifiedEvent
                ):
                    if isinstance(event, t):
                        return True
                return False

            def ignore_events_one_sec(self):
                if not self.wait:
                    self.wait = True
                    t = threading.Thread(target=self.wait_one_sec)
                    t.start()

            def wait_one_sec(self):
                time.sleep(1)
                self.wait = False

            def on_modified(self, event):
                if self.should_reload(event) and not self.wait:
                    print("Some source files have been modified",
                          file=sys.stderr)
                    print("Restarting server...",
                          file=sys.stderr)
                    parent.server_process.kill()
                    self.ignore_events_one_sec()
                    parent.create_subprocess()

        paths = self.paths_to_monitor(conf)

        event_handler = AggressiveEventHandler()

        for path, recurse in paths:
            observer = observers.Observer()
            observer.schedule(
                event_handler,
                path=path,
                recursive=recurse
            )
            observer.start()

        try:
            while True:
                time.sleep(1)
        except KeyboardInterrupt:
            pass



# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import os

from surveil.cmd import surveil_from_nagios
from surveil.tests import base as base_test


class TestSurveilFromNagios(base_test.BaseTestCase):

    def setUp(self):
        self.nagios_config_folder = os.path.join(
            os.path.dirname(os.path.abspath(__file__)),
            'nagios_config'
        )

    def test_surveil_from_nagios_config_cfg(self):
        surveil_cfg = surveil_from_nagios.load_config(
            self.nagios_config_folder
        )

        self.assert_count_equal_backport(
            surveil_cfg,
            [
                ('timeperiods',
                 [
                     {
                         'alias': 'Normal Work Hours',
                         'timeperiod_name': 'workhours',
                         'periods': {
                             'tuesday': '09:00-17:00',
                             'friday': '09:00-17:00',
                             'thursday': '09:00-17:00',
                             'wednesday': '09:00-17:00',
                             'monday': '09:00-17:00'
                         }
                     }
                 ]),
                ('hosts',
                 [
                     {
                         'name': 'generic-host',
                         'custom_fields': {}},
                     {
                         'use': ['generic-host'],
                         'custom_fields': {}},
                     {
                         'use': ['generic-host', 'non-existing-thing'],
                         'contact_groups': ['admins'],
                         'host_name': 'localhost',
                         'check_interval': 324,
                         'address': 'localhost',
                         'custom_fields': {
                             '_custom_yolo': 'sdfsdf'
                         }
                     },

                 ]),
                ('services',
                 [
                     {'host_name': ['test']},
                     {'host_name': ['hai']}
                 ])
            ]
        )

    def test_load_single_file(self):
        other_file_path = os.path.join(
            self.nagios_config_folder,
            'other_file.cfg'
        )

        single_file_config = surveil_from_nagios.load_config(
            other_file_path
        )

        self.assertEqual(
            single_file_config,
            [('services', [{'host_name': ['hai']}])]
        )


# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import pecan
from pecan import rest
import wsmeext.pecan as wsme_pecan

from surveil.api.controllers.v2.status.hosts.services import (
    metrics as services_metrics)
from surveil.api.controllers.v2.status.hosts.services import (
    results as services_check_results)
from surveil.api.datamodel.status import live_service
from surveil.api.handlers.status import live_service_handler
from surveil.common import util


class ServicesController(rest.RestController):

    @pecan.expose()
    def _lookup(self, service_name, *remainder):
        return ServiceController(service_name), remainder


class ServiceController(rest.RestController):

    metrics = services_metrics.MetricsController()
    results = services_check_results.CheckResultsSubController()

    def __init__(self, service_name):
        pecan.request.context['service_name'] = service_name
        self.service_name = service_name

    @util.policy_enforce(['authenticated'])
    @wsme_pecan.wsexpose(live_service.LiveService)
    def get(self):
        """Returns a specific host service."""
        handler = live_service_handler.ServiceHandler(pecan.request)
        service = handler.get(
            pecan.request.context['host_name'],
            self.service_name
        )
        return service



# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""Starter script for the surveil openstack interface"""

from __future__ import print_function

import json
import os
import sys
import threading
import time

import pika
from six.moves import configparser
from surveilclient import client


def main():
    config = configparser.ConfigParser()
    config.read("/etc/surveil/surveil_os_interface.cfg")

    daemon_config = {
        "SURVEIL_API_URL": os.environ.get(
            'SURVEIL_API_URL',
            config.get("surveil-os-interface", "SURVEIL_API_URL")
        ),
        "SURVEIL_AUTH_URL": os.environ.get(
            'SURVEIL_AUTH_URL',
            config.get("surveil-os-interface", "SURVEIL_AUTH_URL")
        ),
        "SURVEIL_VERSION": os.environ.get(
            'SURVEIL_VERSION',
            config.get("surveil-os-interface", "SURVEIL_VERSION")
        ),
        "RABBIT_HOST": os.environ.get(
            'RABBIT_HOST',
            config.get("surveil-os-interface", "RABBIT_HOST")
        ),
        "RABBIT_PORT": int(os.environ.get(
            'RABBIT_PORT',
            config.get("surveil-os-interface", "RABBIT_PORT")
        )
        ),
        "QUEUE": os.environ.get(
            'QUEUE',
            config.get("surveil-os-interface", "QUEUE")
        ),
        "RABBIT_USER": os.environ.get(
            'RABBIT_USER',
            config.get("surveil-os-interface", "RABBIT_USER")
        ),
        "RABBIT_PASSWORD": os.environ.get(
            'RABBIT_PASSWORD',
            config.get("surveil-os-interface", "RABBIT_PASSWORD")
        ),
        "SURVEIL_OS_AUTH_URL": os.environ.get(
            'SURVEIL_OS_AUTH_URL',
            config.get("surveil-os-interface", "SURVEIL_OS_AUTH_URL")
        ),
        "SURVEIL_OS_USERNAME": os.environ.get(
            'SURVEIL_OS_USERNAME',
            config.get("surveil-os-interface", "SURVEIL_OS_USERNAME")
        ),
        "SURVEIL_OS_PASSWORD": os.environ.get(
            'SURVEIL_OS_PASSWORD',
            config.get("surveil-os-interface", "SURVEIL_OS_PASSWORD")
        ),
        "SURVEIL_OS_TENANT_NAME": os.environ.get(
            'SURVEIL_OS_TENANT_NAME',
            config.get("surveil-os-interface", "SURVEIL_OS_TENANT_NAME")
        ),
        "SURVEIL_DEFAULT_TAGS": os.environ.get(
            'SURVEIL_DEFAULT_TAGS',
            config.get("surveil-os-interface", "SURVEIL_DEFAULT_TAGS")
        ),
        "SURVEIL_NETWORK_LABEL": os.environ.get(
            'SURVEIL_NETWORK_LABEL',
            config.get("surveil-os-interface", "SURVEIL_NETWORK_LABEL")
        ),
    }

    if (daemon_config["RABBIT_USER"] is not None
            and daemon_config["RABBIT_PASSWORD"] is not None):
        id = pika.credentials.PlainCredentials(
            daemon_config["RABBIT_USER"],
            daemon_config["RABBIT_PASSWORD"]
        )

        connection = pika.BlockingConnection(
            pika.ConnectionParameters(
                host=daemon_config["RABBIT_HOST"],
                port=daemon_config["RABBIT_PORT"],
                credentials=id
            )
        )
    else:
        connection = pika.BlockingConnection(
            pika.ConnectionParameters(
                host=daemon_config["RABBIT_HOST"],
                port=daemon_config["RABBIT_PORT"]
            )
        )

    channel = connection.channel()
    channel.queue_declare(queue=daemon_config["QUEUE"])

    threads = []
    compt_thread = []

    def reload_config_threads():
        c = client.Client(daemon_config["SURVEIL_API_URL"],
                          auth_url=daemon_config["SURVEIL_AUTH_URL"],
                          version=daemon_config["SURVEIL_VERSION"])
        while True:
            time.sleep(30)
            if compt_thread:
                c.config.reload_config()
                del (compt_thread[:])

    def join_finished_threads():
        while True:
            time.sleep(1)
            todel = []
            for t in threads:
                if not t.isAlive():
                    t.join()
                    todel.append(t)
            for t in todel:
                threads.remove(t)

    joiner_thread = threading.Thread(target=join_finished_threads)
    joiner_thread.daemon = True
    joiner_thread.start()

    reload_config_thread = threading.Thread(target=reload_config_threads)
    reload_config_thread.daemon = True
    reload_config_thread.start()

    def process_instance_create_start(event, sclient):
            custom_fields = {
                "_OS_AUTH_URL": daemon_config["SURVEIL_OS_AUTH_URL"],
                "_OS_TENANT_NAME": daemon_config["SURVEIL_OS_TENANT_NAME"],
                "_OS_USERNAME": daemon_config["SURVEIL_OS_USERNAME"],
                "_OS_PASSWORD": daemon_config["SURVEIL_OS_PASSWORD"],
                "_OS_INSTANCE_ID": event['payload']['instance_id'],
                "_OS_INSTANCE_METADATA": json.dumps(
                    event['payload']['metadata']
                ),
            }

            surveil_metadata_custom_fields = event['payload']['metadata'].get(
                'surveil_custom_fields',
                None
            )

            #  Custom fields
            if surveil_metadata_custom_fields is not None:
                try:
                    custom_fields.update(
                        json.loads(surveil_metadata_custom_fields)
                    )
                except ValueError:
                    print("Could not load json %s" %
                          surveil_metadata_custom_fields,
                          file=sys.stderr)

            # Tags
            instance_tags = daemon_config["SURVEIL_DEFAULT_TAGS"].split(',')
            surveil_metadata_tags = event['payload']['metadata'].get(
                'surveil_tags',
                None
            )
            if surveil_metadata_tags is not None:
                instance_tags += surveil_metadata_tags.split(',')

            sclient.config.hosts.create(
                host_name=event['payload']['hostname'],
                address=event['payload']['hostname'],
                use=instance_tags,
                custom_fields=custom_fields
            )

    def process_instance_create_end(event, sclient):
        #  Get the ip address
        addresses = event['payload'].get('fixed_ips', [])
        for addr in addresses:
            if (addr.get('label', '') ==
                    daemon_config['SURVEIL_NETWORK_LABEL']):
                sclient.config.hosts.update(
                    event['payload']['hostname'],
                    address=addr['address']
                )
                break

    def process_instance_delete_end(sclient, event):
        sclient.config.hosts.delete(event['payload']['hostname'])

    def process_event(body):
        sclient = client.Client(daemon_config["SURVEIL_API_URL"],
                                auth_url=daemon_config["SURVEIL_AUTH_URL"],
                                version=daemon_config["SURVEIL_VERSION"])

        #  Load the event
        event = json.loads(body)

        #  Process the event
        try:
            if event['event_type'] == 'compute.instance.create.start':
                process_instance_create_start(event, sclient)
            elif event['event_type'] == 'compute.instance.create.end':
                process_instance_create_end(event, sclient)
            elif event['event_type'] == 'compute.instance.delete.end':
                process_instance_delete_end(sclient, event)
        except Exception as e:
            print("Could not process event %s" % e, file=sys.stderr)

    def callback(ch, method, properties, body):
        t = threading.Thread(target=process_event, args=(body,))
        t.daemon = True
        t.start()
        threads.append(t)
        compt_thread.append(t)

    channel.basic_consume(callback, queue=daemon_config["QUEUE"], no_ack=True)
    channel.start_consuming()



# Copyright 2015 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import wsme

from surveil.api.datamodel import types


class Paging(types.Base):

    size = wsme.wsattr(int, mandatory=True)
    """Size of the result."""

    page = wsme.wsattr(int, mandatory=True)
    """Page number."""

    @classmethod
    def sample(cls):
        return cls(
            page=1,
            size=100
        )



# Copyright 2015 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from surveil.api.datamodel.status import event
from surveil.api.handlers import handler
from surveil.api.handlers.status import influxdb_query


class EventHandler(handler.Handler):
    """Fulfills a request on the events resource."""

    def get_all(self, live_query=None):
        """Return all logs."""
        influx_client = self.request.influxdb_client
        query = influxdb_query.build_influxdb_query(live_query, "EVENT",
                                                    multiple_series=True)
        return influxdb_query.paging(influx_client.query(query), event.Event, live_query)


# Copyright 2015 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import pecan
from pecan import rest
import requests
import wsmeext.pecan as wsme_pecan

from surveil.api.datamodel import checkresult
from surveil.common import util


class CheckResultsSubController(rest.RestController):

    @util.policy_enforce(['authenticated'])
    @wsme_pecan.wsexpose(body=checkresult.CheckResult, status_code=204)
    def post(self, data):
        """Submit a new check result.

        :param data: a check result within the request body.
        """
        result = data.as_dict()
        result['host_name'] = pecan.request.context['host_name']

        requests.post(
            pecan.request.ws_arbiter_url + "/push_check_result",
            data=result
        )
